"""High-level gRPC channel manager for the Nebius Python SDK."""

import sys
from asyncio import (
    FIRST_COMPLETED,
    AbstractEventLoop,
    CancelledError,
    Task,
    create_task,
    gather,
    get_event_loop,
    iscoroutine,
    new_event_loop,
    run_coroutine_threadsafe,
    sleep,
    wait,
)
from collections.abc import Awaitable, Coroutine, Sequence
from inspect import isawaitable
from logging import getLogger
from pathlib import Path
from typing import Any, TextIO, TypeVar

from google.protobuf.message import Message
from grpc import (
    CallCredentials,
    ChannelConnectivity,
    ChannelCredentials,
    Compression,
    ssl_channel_credentials,
)
from grpc.aio import Channel as GRPCChannel
from grpc.aio._base_call import UnaryUnaryCall
from grpc.aio._base_channel import (
    StreamStreamMultiCallable,
    StreamUnaryMultiCallable,
    UnaryStreamMultiCallable,
    UnaryUnaryMultiCallable,
)
from grpc.aio._channel import (
    insecure_channel,  # type: ignore[unused-ignore]
    secure_channel,  # type: ignore[unused-ignore]
)
from grpc.aio._interceptor import ClientInterceptor
from grpc.aio._typing import (
    ChannelArgumentType,
    DeserializingFunction,
    MetadataType,
    SerializingFunction,
)

from nebius.aio.abc import GracefulInterface
from nebius.aio.authorization.authorization import Provider as AuthorizationProvider
from nebius.aio.authorization.token import TokenProvider
from nebius.aio.cli_config import Config as ConfigReader
from nebius.aio.idempotency import IdempotencyKeyInterceptor
from nebius.aio.service_descriptor import ServiceStub, from_stub_class
from nebius.aio.token import exchangeable, renewable
from nebius.aio.token.static import Bearer as StaticTokenBearer
from nebius.aio.token.static import EnvBearer
from nebius.aio.token.token import Bearer as TokenBearer
from nebius.aio.token.token import Token
from nebius.api.nebius.common.v1.operation_service_pb2_grpc import (
    OperationServiceStub,
)
from nebius.api.nebius.common.v1alpha1.operation_service_pb2_grpc import (
    OperationServiceStub as OperationServiceStubDeprecated,
)
from nebius.base.constants import DOMAIN
from nebius.base.error import SDKError
from nebius.base.methods import service_from_method_name
from nebius.base.options import COMPRESSION, INSECURE, pop_option
from nebius.base.resolver import Chain, Conventional, Resolver, TemplateExpander
from nebius.base.service_account.service_account import (
    Reader as ServiceAccountReader,
)
from nebius.base.service_account.service_account import (
    TokenRequester as TokenRequestReader,
)
from nebius.base.tls_certificates import get_system_certificates
from nebius.base.version import version

from .base import AddressChannel, ChannelBase

logger = getLogger(__name__)

Req = TypeVar("Req", bound=Message)
Res = TypeVar("Res", bound=Message)

T = TypeVar("T")


class LoopError(SDKError):
    """Exception raised when a synchronous helper is used incorrectly with
    an asyncio event loop.

    This error is raised when the code attempts to perform a synchronous
    operation (for example, calling :meth:`Channel.run_sync`) while the
    targeted asyncio event loop is already running in the current thread.

    The exception subclasses :class:`SDKError` so callers
    catching SDK-related errors will also catch this condition.

    ``LoopError`` does not add any new behaviour beyond the base error; it
    serves only to provide a more specific error type for loop misuse.
    """


class ChannelClosedError(SDKError):
    """Raised when an operation is attempted on a closed :class:`Channel`.

    This indicates that :meth:`Channel.close` (or :meth:`Channel.sync_close`)
    was previously called and the channel no longer accepts requests or
    returns channel objects.
    """


class NebiusUnaryUnaryMultiCallable(UnaryUnaryMultiCallable[Req, Res]):  # type: ignore[unused-ignore,misc]
    """A small callable wrapper that binds RPC calls to a Channel-managed
    address channel.

    Instances of this class behave like a gRPC :class:`UnaryUnaryMultiCallable`
    but ensure that the underlying transport channel is obtained from the
    SDK :class:`Channel` pool and returned (or discarded) when the RPC
    completes.
    """

    def __init__(
        self,
        channel: "Channel",
        method: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
    ) -> None:
        """Create a callable wrapper that returns requests bound to an
        :class:`AddressChannel` from the SDK :class:`Channel`.

        :param channel: The SDK :class:`Channel` instance used to obtain a
            transport channel for the RPC.
        :param method: Full RPC method string (``'/package.service/Method'``).
        :param request_serializer: Optional serializer used by gRPC.
        :param response_deserializer: Optional deserializer used by gRPC.
        """
        super().__init__()
        self._channel = channel
        self._method = method
        self._request_serializer = request_serializer
        self._response_deserializer = response_deserializer

    def __call__(
        self,
        request: Req,
        *,
        timeout: float | None = None,
        metadata: MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> UnaryUnaryCall[Req, Res]:
        """Invoke the underlying unary-unary RPC on an address channel.

        This method resolves the concrete address for ``self._method`` and
        requests an :class:`AddressChannel` from the parent :class:`Channel`.
        The returned :class:`grpc.aio.UnaryUnaryCall` has a done-callback
        attached that returns or discards the address channel back to the
        channel pool when the RPC completes.

        :param request: The protobuf request message to send.
        :param timeout: Optional per-call timeout in seconds.
        :param metadata: Optional gRPC metadata to send with the request.
        :param credentials: Optional per-call call-credentials.
        :param wait_for_ready: Optional gRPC wait_for_ready flag.
        :param compression: Optional gRPC compression setting.
        :return: A :class:`grpc.aio.UnaryUnaryCall` representing the in-flight
            RPC. The caller may await or add callbacks to the returned object.
        """

        ch = self._channel.get_channel_by_method(self._method)
        ret = ch.channel.unary_unary(  # type: ignore[unused-ignore,call-arg,assignment,misc]
            self._method,
            self._request_serializer,
            self._response_deserializer,
        )(  # type: ignore[unused-ignore,call-arg,assignment,misc]
            request,
            timeout=timeout,
            metadata=metadata,  # type: ignore
            credentials=credentials,
            wait_for_ready=wait_for_ready,
            compression=compression,
        )

        def return_channel(cb_arg: Any) -> None:
            """Callback executed when the RPC finishes.

            The callback returns or discards the address channel back to the
            channel pool. If the channel manager has already been closed it
            attempts a short synchronous close of the underlying channel.
            """
            nonlocal ch
            logger.debug(f"Done with call {self=}, {cb_arg=}")
            try:
                self._channel.discard_channel(ch)
            except ChannelClosedError:
                self._channel.run_sync(ch.channel.close(None), 0.1)
                # pass
            ch = None  # type: ignore

        ret.add_done_callback(return_channel)
        return ret  # type: ignore[unused-ignore,return-value]


class NoCredentials:
    """Marker type used to explicitly disable authorization.

    When this value is supplied as the ``credentials`` parameter to
    :class:`Channel`, the channel will not attempt to acquire or attach any
    authorization tokens for outgoing requests.
    """


Credentials = (
    AuthorizationProvider
    | TokenBearer
    | TokenRequestReader
    | NoCredentials
    | Token
    | str
    | None
)


def _wrap_awaitable(awaitable: Awaitable[T]) -> Coroutine[Any, Any, T]:
    """Ensure the provided awaitable is a coroutine object.

    gRPC helper functions in this module accept both coroutine objects and
    other awaitable types (for example :class:`asyncio.Future`). This function
    normalizes them into a coroutine so that they can be wrapped in an
    :class:`asyncio.Task` safely.

    :param awaitable: Any awaitable or coroutine-like object.
    :return: A coroutine object ready to be scheduled.
    :raises TypeError: If the argument is not awaitable.
    """

    if iscoroutine(awaitable):
        return awaitable
    if not isawaitable(awaitable):
        raise TypeError(
            "An asyncio.Future, a coroutine or an awaitable is "
            + f"required, {type(awaitable)} given"
        )

    async def wrap() -> T:
        """Adapter coroutine that awaits the supplied awaitable and returns
        its result.

        This small wrapper is used to convert generic awaitable objects into
        a true coroutine so they can be scheduled as an :class:`asyncio.Task`.
        """
        return await awaitable

    return wrap()


async def _run_awaitable_with_timeout(
    f: Awaitable[T],
    timeout: float | None = None,
) -> T:
    """Run an awaitable with an optional wall-clock timeout.

    The function creates an :class:`asyncio.Task` from the provided awaitable
    and, if a timeout is supplied, a short timer task. It waits for the first
    task to finish. If the timer completes first the awaited task is
    cancelled and a :class:`TimeoutError` is raised. Exceptions raised by the
    awaited task are propagated.

    :param f: The awaitable to run.
    :param timeout: Optional timeout in seconds. If ``None`` the awaitable is
        allowed to run indefinitely.
    :return: The awaited result.
    :raises TimeoutError: If the awaitable did not finish before the timeout.
    """

    task = Task(_wrap_awaitable(f), name=f"Task for {f=}")
    tasks: list[Task[Any]] = list[Task[Any]]([task])
    if timeout is not None:
        timer = Task(sleep(timeout), name=f"Timer for {f=}")
        tasks.append(timer)
    done, pending = await wait(
        tasks,
        return_when=FIRST_COMPLETED,
    )
    for p in pending:
        logger.debug(f"Canceling pending task {p}")
        p.cancel()
    await gather(*pending, return_exceptions=True)
    try:
        if task.exception() is not None:
            if task not in done:
                raise TimeoutError("Awaitable timed out") from task.exception()
            raise task.exception()  # type: ignore
    except CancelledError as e:
        if task not in done:
            raise TimeoutError("Awaitable timed out") from e
        raise e
    return task.result()


def set_user_agent_option(
    user_agent: str, options: ChannelArgumentType | None
) -> ChannelArgumentType:
    """
    Set or override the ``grpc.primary_user_agent`` channel option.
    This helper appends the provided user-agent string to the ``options``
    sequence, which is passed to gRPC when creating channels. If the
    ``grpc.primary_user_agent`` option is already present in ``options``,
    it will be replaced with the new value.

    :param user_agent: The user-agent string to set.
    :type user_agent: str
    :param options: Existing channel options, if any.
    :type options: optional list of ``(str, Any)`` tuples
    :return: The updated channel options including the user-agent.
    :rtype: list of ``(str, Any)`` tuples
    """
    options = list(options or [])
    options.append(("grpc.primary_user_agent", user_agent))
    return options


class Channel(ChannelBase):  # type: ignore[unused-ignore,misc]
    """A high-level gRPC channel manager used by the SDK.

    Responsibilities and behavior
    ==============================

    - Resolve service names to addresses and create underlying gRPC channels for
        those addresses.
    - Maintain a small pool of free channels per address to reuse connections and
        reduce churn (see ``max_free_channels_per_address`` init param).
    - Provide helpers to attach authorization credentials and manage token
        providers, including synchronous and asynchronous token acquisition via
        :meth:`get_token` and :meth:`get_token_sync`.
    - Expose convenience wrappers for unary-unary RPCs that ensure the address
        channel is returned to the pool when the RPC completes (see
        :class:`NebiusUnaryUnaryMultiCallable`).
    - Offer both async and sync usage models. Use the async context manager
        (``async with Channel(...)``) or call ``close()``/``sync_close()`` to
        gracefully shut down resources.

    Important notes
    ---------------

    - The channel owns a configurable asyncio event loop (optionally provided at
        construction). When calling ``run_sync`` from within an already running
        loop, a :class:`LoopError` is raised unless a separate loop was supplied at
        initialization.
    - The class cooperates with several SDK subsystems (token bearers,
        authorization providers, idempotency interceptor, and a resolver) and
        wires them together during initialization.
    - Public helper methods include: ``get_token``/``get_token_sync``,
        ``run_sync``, ``bg_task``, ``get_channel_by_method``, and
        ``create_address_channel``.

    Usage example
    =============

    Async usage (recommended)::

        async with Channel(...) as channel:
            # use channel or pass it to generated service clients
            pass

    Synchronous usage::

        channel = Channel(...)
        try:
            channel.run_sync(some_coroutine())
        finally:
            channel.sync_close()

    :ivar user_agent: The user-agent string used by channels created by
        this Channel instance.

    :param resolver:
        Optional custom :class:`Resolver` used to resolve service names to concrete
        addresses. If omitted a :class:`Conventional` resolver is used. If
        provided, it will be chained with the built-in resolver so both
        can be consulted.
    :type resolver: optional :class:`Resolver`

    :param substitutions:
        Optional mapping of template substitutions applied to resolved
        addresses. The construct inserts ``{"{domain}": domain}`` and
        then updates it with this mapping. Typical use is to override
        domain placeholders in generated service addresses.
    :type substitutions: optional dict[from, to]

    :param user_agent_prefix:
        Optional string prepended to the default SDK user-agent. The
        final user-agent string follows the pattern
        ``"<user_agent_prefix> nebius-python-sdk/<version> (python/X.Y.Z)"``.
        Recommended format:
        ``"my-app/1.0 (dependency-to-track/version; other-dependency)"``.
    :type user_agent_prefix: optional str

    :param domain:
        Optional domain to substitute into service addresses. When not
        provided the constructor will try to obtain it from the provided
        ``config_reader`` via ``config_reader.endpoint()``, and fall back
        to the package-level ``DOMAIN`` constant if still unset.
    :type domain: optional str

    :param options:
        Global channel options passed to gRPC when creating address
        channels. This should follow the ``ChannelArgumentType``
        shape (sequence of key/value tuples).
    :type options: optional list of tuple[str, Any]

    :param interceptors:
        Global list of gRPC :class:`ClientInterceptor`
        instances that will be applied to all address channels. An
        idempotency-key interceptor is added by default; pass a list to
        extend or override additional behavior.
    :type interceptors: optional list of :class:`ClientInterceptor`

    :param address_options:
        Optional mapping from a resolved address to per-address channel
        options. Each value must follow the ``ChannelArgumentType``
        shape (sequence of key/value tuples). If omitted
        an empty mapping is used.
    :type address_options: optional mapping address -> list of ``tuple[str, Any]``

    :param address_interceptors:
        Optional mapping from a resolved address to a sequence of
        per-address interceptors. Per-address interceptors are invoked
        in addition to the global interceptors.
    :type address_interceptors: optional mapping address ->
        Sequence[ClientInterceptor]

    :param credentials:
        Credentials can be provided in several forms:

        - ``None`` (default): attempts to read credentials from
            ``credentials_file_name``, then from provided service account
            fields, then from ``config_reader.get_credentials(...)``, and
            finally falls back to an environment-backed bearer
            (:class:`nebius.aio.token.static.EnvBearer`).
        - ``str`` or :class:`Token`: treated as a
            static token and wrapped with a static bearer.
        - :class:`TokenBearer` to use an existing token bearer as-is.
        - :class:`TokenRequester` to exchange tokens on demand.
        - :class:`AuthorizationProvider`: an explicit authorization provider
            (used rarely by advanced users).
        - :class:`NoCredentials`: disables authorization entirely.

        Unsupported types raise :class:`SDKError`.
    :type credentials: token in form of string or :class:`Token`, or classes
        :class:`TokenBearer`, :class:`TokenRequester`,
        :class:`AuthorizationProvider`, :class:`NoCredentials`

    :param service_account_id:
        Service account ID used when a private key file is supplied
        directly (alternate to using ``credentials_file_name``). See the
        README for examples. If ``credentials`` is provided explicitly
        this parameter is ignored.
    :type service_account_id: optional str

    :param service_account_public_key_id:
        Public key ID corresponding to the private key file used for
        service-account authentication, as described in the README. If
        ``credentials`` is provided explicitly this parameter is ignored.
    :type service_account_public_key_id: optional str

    :param service_account_private_key_file_name:
        Path to a PEM private key file. When provided with the key ID and service
        account ID fields above, the constructor wraps it in a service-account
        reader.
    :type service_account_private_key_file_name: optional str or :class:`Path`

    :param credentials_file_name:
        Path to a credentials JSON file containing service-account
        information. If supplied this takes precedence over other implicit
        credential discovery (unless ``credentials`` is explicitly
        provided).
    :type credentials_file_name: optional str or :class:`Path`

    :param config_reader:
        Optional :class:`nebius.aio.cli_config.Config` instance used to
        populate defaults like domain, default parent ID, and to obtain
        credentials via the CLI-style configuration.
    :type config_reader: optional :class:`ConfigReader`

    :param tls_credentials:
        Optional gRPC channel TLS credentials (:class:`ChannelCredentials`).
        If omitted the constructor will load system root certificates via
        :func:`nebius.base.tls_certificates.get_system_certificates` and
        create an SSL channel credentials object.
    :type tls_credentials: optional :class:`ChannelCredentials`

    :param event_loop:
        Optional asyncio event loop to be owned by this Channel. When a
        loop is provided, synchronous helpers such as :meth:`run_sync`
        may be called from other threads without raising
        :class:`LoopError`. If not provided the Channel will lazily
        create its own loop when a synchronous call is made.
    :type event_loop: optional :class:`AbstractEventLoop`

    :param max_free_channels_per_address:
        Number of free underlying gRPC channels to keep in the pool per
        resolved address. Defaults to 2. Lower values reduce resource
        usage but increase connection churn; larger values raise resource
        consumption.
    :type max_free_channels_per_address: optional int

    :param parent_id:
        Optional parent ID which will be automatically applied to many
        requests when left empty by the caller. If not provided and a
        ``config_reader`` is supplied the constructor will attempt to use
        ``config_reader.parent_id``. An explicit empty string is treated
        as an error.
    :type parent_id: optional str

    :param federation_invitation_writer:
        Optional file-like writer passed to the config reader to display
        the URL for federation authentication during interactive credential
        acquisition.
    :type federation_invitation_writer: optional :class:`TextIO`

    :param federation_invitation_no_browser_open:
        When using the config reader, set to ``True`` to avoid opening a web
        browser during interactive federation flows. Defaults to ``False``.
    :type federation_invitation_no_browser_open: optional bool
    """

    def __init__(
        self,
        *,
        resolver: Resolver | None = None,
        substitutions: dict[str, str] | None = None,
        user_agent_prefix: str | None = None,
        domain: str | None = None,
        options: ChannelArgumentType | None = None,
        interceptors: Sequence[ClientInterceptor] | None = None,
        address_options: dict[str, ChannelArgumentType] | None = None,
        address_interceptors: dict[str, Sequence[ClientInterceptor]] | None = None,
        credentials: Credentials = None,
        service_account_id: str | None = None,
        service_account_public_key_id: str | None = None,
        service_account_private_key_file_name: str | Path | None = None,
        credentials_file_name: str | Path | None = None,
        config_reader: ConfigReader | None = None,
        tls_credentials: ChannelCredentials | None = None,
        event_loop: AbstractEventLoop | None = None,
        max_free_channels_per_address: int = 2,
        parent_id: str | None = None,
        federation_invitation_writer: TextIO | None = None,
        federation_invitation_no_browser_open: bool = False,
    ) -> None:
        """
        Construct a new Channel instance.

        This constructor wires together the SDK's gRPC channel management,
        credential providers, resolvers, TLS configuration and interceptors.

        The Channel is responsible for resolving logical service names to
        transport addresses, creating and pooling underlying gRPC channels,
        and exposing helpers for synchronous and asynchronous usage.

        :raises SDKError:
            Raised for unsupported credential types or if ``parent_id`` is an
            explicitly empty string.

        Notes
        -----
        - The constructor performs several discovery steps for credentials in
          the following precedence order when ``credentials`` is ``None``:
          1. ``credentials_file_name`` reader
          2. service-account PEM reader (when id/key args are provided)
          3. ``config_reader.get_credentials(...)``
          4. environment-backed bearer (:class:`EnvBearer`)

        - Token readers are wrapped into exchangeable/renewable bearers so
          that token refresh happens transparently in the background and the
          Channel adds the bearer to its graceful shutdown set to ensure
          background tasks are cleaned up on :meth:`close`.

        Examples
        --------
        Typical, minimal construction that reads token from environment:

        >>> channel = Channel()

        Using explicit static token:

        >>> channel = Channel(credentials="MY_TOKEN")

        Creating channel from CLI config and a custom resolver:

        >>> from nebius.aio.cli_config import Config
        >>> channel = Channel(config_reader=Config(), resolver=my_resolver)

        """

        import nebius.api.nebius.iam.v1.token_exchange_service_pb2  # type: ignore[unused-ignore] # noqa: F401 - load for registration
        import nebius.api.nebius.iam.v1.token_exchange_service_pb2_grpc  # type: ignore[unused-ignore] # noqa: F401 - load for registration

        if domain is None:
            if config_reader is not None:
                domain = config_reader.endpoint()

            if domain is None or domain == "":
                domain = DOMAIN

        substitutions_full = dict[str, str]()
        substitutions_full["{domain}"] = domain
        if substitutions is not None:
            substitutions_full.update(substitutions)

        self._max_free_channels_per_address = max_free_channels_per_address

        self._gracefuls = set[GracefulInterface]()
        self._tasks = set[Task[Any]]()

        self._resolver: Resolver = Conventional()
        if resolver is not None:
            self._resolver = Chain(resolver, self._resolver)
        self._resolver = TemplateExpander(substitutions_full, self._resolver)
        if tls_credentials is None:
            root_ca = get_system_certificates()
            with open(root_ca, "rb") as f:
                trusted_certs = f.read()
            tls_credentials = ssl_channel_credentials(root_certificates=trusted_certs)
        self._tls_credentials = tls_credentials

        self._free_channels = dict[str, list[GRPCChannel]]()
        self._methods = dict[str, str]()
        self.user_agent = "nebius-python-sdk/" + version
        self.user_agent += f" (python/{sys.version_info.major}.{sys.version_info.minor}"
        self.user_agent += f".{sys.version_info.micro})"

        if user_agent_prefix is not None:
            self.user_agent = f"{user_agent_prefix} {self.user_agent}"

        if interceptors is None:
            interceptors = []
        self._global_options = options or []
        self._global_interceptors: list[ClientInterceptor] = [
            IdempotencyKeyInterceptor()
        ]
        self._global_interceptors.extend(interceptors)

        if address_options is None:
            address_options = dict[str, ChannelArgumentType]()
        if address_interceptors is None:
            address_interceptors = dict[str, Sequence[ClientInterceptor]]()
        self._address_options = address_options
        self._address_interceptors = address_interceptors

        self._global_interceptors_inner: list[ClientInterceptor] = []

        self._parent_id = parent_id
        if self._parent_id is None and config_reader is not None:
            from .cli_config import NoParentIdError

            try:
                self._parent_id = config_reader.parent_id
            except NoParentIdError:
                pass
        if self._parent_id == "":
            raise SDKError("Parent id is empty")

        self._token_bearer: TokenBearer | None = None
        self._authorization_provider: AuthorizationProvider | None = None
        if credentials is None:
            if credentials_file_name is not None:
                from nebius.base.service_account.credentials_file import (
                    Reader as CredentialsFileReader,
                )

                credentials = CredentialsFileReader(credentials_file_name)
            elif (
                service_account_id is not None
                and service_account_private_key_file_name is not None
                and service_account_public_key_id is not None
            ):
                from nebius.base.service_account.pk_file import Reader as PKFileReader

                credentials = PKFileReader(
                    service_account_private_key_file_name,
                    service_account_public_key_id,
                    service_account_id,
                )
            elif config_reader is not None:
                credentials = config_reader.get_credentials(
                    self,
                    writer=federation_invitation_writer,
                    no_browser_open=federation_invitation_no_browser_open,
                )
            else:
                credentials = EnvBearer()
        if isinstance(credentials, str) or isinstance(credentials, Token):
            credentials = StaticTokenBearer(credentials)
        if isinstance(credentials, ServiceAccountReader):
            from nebius.aio.token.service_account import ServiceAccountBearer

            credentials = ServiceAccountBearer(
                credentials,
                self,
            )
        if isinstance(credentials, TokenRequestReader):
            exchange = exchangeable.Bearer(credentials, self)
            cache = renewable.Bearer(exchange)
            credentials = cache
        if isinstance(credentials, TokenBearer):
            self._gracefuls.add(credentials)
            self._token_bearer = credentials
            credentials = TokenProvider(credentials)
        if isinstance(credentials, AuthorizationProvider):
            self._authorization_provider = credentials
        elif not isinstance(credentials, NoCredentials):  # type: ignore[unused-ignore]
            raise SDKError(f"credentials type is not supported: {type(credentials)}")

        self._event_loop = event_loop
        self._closed = False

    def get_authorization_provider(self) -> AuthorizationProvider | None:
        """Return the configured :class:`AuthorizationProvider`.

        :return: The authorization provider instance if any authorization
            mechanism was configured; otherwise ``None``.
        :rtype: :class:`AuthorizationProvider` or None
        """
        return self._authorization_provider

    async def get_token(
        self,
        timeout: float | None,
        options: dict[str, str] | None = None,
    ) -> Token:
        """Asynchronously fetch an authorization :class:`Token`.

        This helper delegates to the configured token bearer and performs any
        necessary refresh or exchange logic implemented by the bearer, if any was
        configured. If no bearer was configured, the method raises
        :class:`SDKError`.

        :param timeout: Maximum time in seconds to wait for a token. If
            ``None`` the operation may block indefinitely according to the
            bearer semantics.
        :type timeout: optional float
        :param options: Optional mapping of string options passed to the
            underlying token receiver.
        :type options: optional ``dict[str, str]``
        :return: A :class:`Token` instance containing the access token.
        :rtype: :class:`Token`
        :raises SDKError: If no token bearer was configured on the channel.
        """

        if self._token_bearer is None:
            raise SDKError("Token bearer is not set")
        receiver = self._token_bearer.receiver()
        return await receiver.fetch(
            timeout=timeout,
            options=options,
        )

    def get_token_sync(
        self,
        timeout: float | None,
        options: dict[str, str] | None = None,
    ) -> Token:
        """Synchronously fetch an authorization :class:`Token`.

        This method is a convenience wrapper around :meth:`get_token` that
        runs the async fetch on the channel's owned event loop and blocks the
        calling thread until the token is available or the timeout expires.

        A small grace period is added to the supplied timeout to allow the
        internal token bearer shutdown logic to complete during immediate
        handoff.

        :param timeout: Maximum time in seconds to wait for a token; may be
            ``None`` to wait indefinitely.
        :type timeout: optional float
        :param options: Optional mapping of string options passed to the
            underlying token receiver.
        :type options: optional ``dict[str, str]``
        :return: A :class:`Token` instance.
        :rtype: :class:`Token`
        :raises TimeoutError: If the token could not be obtained within the
            supplied timeout.
        """

        timeout_sync = timeout
        if timeout_sync is not None:
            timeout_sync += 0.2  # 200 ms for graceful shutdown
        return self.run_sync(
            self.get_token(timeout, options),
            timeout_sync,
        )

    def parent_id(self) -> str | None:
        """Return the channel-wide default parent ID used for certain requests.

        Some SDK methods automatically populate a ``parent_id`` field when
        missing using this channel-level default. The value may be ``None`` if not
        configured.

        :return: The configured parent ID or ``None``.
        :rtype: str | None
        """

        return self._parent_id

    def bg_task(self, coro: Awaitable[T]) -> Task[None]:
        """Run a coroutine in the background and log uncaught exceptions.

        The returned task is automatically tracked by the channel and will be
        cancelled during :meth:`close`. Any non-cancellation exception is
        logged to the module logger.

        :param coro: An awaitable object to run in the background.
        :return: The created :class:`asyncio.Task`.
        """

        async def wrapper() -> None:
            """Internal runner that awaits the provided coroutine and logs
            uncaught exceptions.

            Cancellation errors are ignored since they are part of normal
            shutdown semantics.
            """
            try:
                await coro
            except CancelledError:
                pass
            except Exception as e:
                logger.error("Unhandled exception in Channel.bg_task", exc_info=e)

        ret = create_task(wrapper(), name=f"Channel.bg_task for {coro}")
        ret.add_done_callback(lambda x: self._tasks.discard(x))
        self._tasks.add(ret)
        return ret

    def run_sync(self, awaitable: Awaitable[T], timeout: float | None = None) -> T:
        """Run an awaitable to completion on the channel's event loop.

        This helper supports two usage patterns:

        - If the channel was initialized with an explicit event loop, the
          awaitable will be scheduled on that loop and this function may be
          safely called from another thread.
        - If the channel has no provided loop, an internal loop is created or
          reused and ``run_until_complete`` is used. Calling this from a
          running loop without providing a separate loop raises
          :class:`LoopError`.

        :param awaitable: The awaitable to run to completion.
        :param timeout: Optional wall-clock timeout in seconds. If provided
            and the awaitable does not complete within the timeout a
            :class:`TimeoutError` is raised.
        :type timeout: optional float
        :return: The awaitable's result.
        :raises LoopError: When called synchronously inside a running loop
            and no separate event loop was provided at construction.
        """

        loop_provided = self._event_loop is not None
        if self._event_loop is None:
            try:
                self._event_loop = get_event_loop()
            except RuntimeError:
                self._event_loop = new_event_loop()

        if self._event_loop.is_running():
            if loop_provided:
                try:
                    if get_event_loop() == self._event_loop:
                        raise LoopError(
                            "Provided loop is equal to current thread's "
                            "loop. Either use async/await or provide "
                            "another loop at the SDK initialization."
                        )
                except RuntimeError:
                    pass
                return run_coroutine_threadsafe(
                    _run_awaitable_with_timeout(awaitable, timeout),
                    self._event_loop,
                ).result()
            else:
                raise LoopError(
                    "Synchronous call inside async context. Either use "
                    "async/await or provide a safe and separate loop "
                    "to run at the SDK initialization."
                )

        return self._event_loop.run_until_complete(
            _run_awaitable_with_timeout(awaitable, timeout)
        )

    def sync_close(self, timeout: float | None = None) -> None:
        """Synchronously close the channel and wait for graceful shutdown.

        This is a convenience wrapper around :meth:`close` that blocks until
        the shutdown completes or the optional timeout elapses.

        :param timeout: Optional timeout in seconds for the shutdown.
        :type timeout: optional float
        :raises TimeoutError: If the shutdown did not complete within the
            supplied timeout.
        """

        return self.run_sync(self.close(), timeout)

    async def close(self, grace: float | None = None) -> None:
        """Gracefully close the channel and all associated background work.

        The channel will stop handing out address channels and will attempt to
        close any pooled gRPC channels and all registered ``GracefulInterface``
        objects (for example token bearers). Background tasks started via
        :meth:`bg_task` are cancelled. Any exceptions raised during shutdown
        are logged.

        :param grace: Optional per-transport grace period passed to underlying
            channel close methods.
        :type grace: optional float
        """

        self._closed = True
        awaits = list[Coroutine[Any, Any, Any]]()
        for chans in self._free_channels.values():
            for chan in chans:
                awaits.append(chan.close(grace))
        for graceful in self._gracefuls:
            awaits.append(graceful.close(grace))
        for task in self._tasks:
            task.cancel()
        rets = await gather(*awaits, *self._tasks, return_exceptions=True)
        for ret in rets:
            if isinstance(ret, BaseException) and not isinstance(ret, CancelledError):
                logger.error(f"Error while graceful shutdown: {ret}", exc_info=ret)

    def get_corresponding_operation_service(
        self,
        service_stub_class: type[ServiceStub],
    ) -> OperationServiceStub:
        """Return an operations service stub for the same address as a
        generated service stub.

        Many long-running operations are associated with the service that
        initiated them. This helper resolves the service address for the
        provided generated stub class and returns an instantiated
        :class:`OperationServiceStub` bound to the transport channel for that address.

        :param service_stub_class: Generated gRPC service stub class (the SDK service
            descriptor type).
        :return: An operations service stub bound to the same backend used by
            the provided service.
        :rtype: :class:`OperationServiceStub`
        """

        addr = self.get_addr_from_stub(service_stub_class)
        chan = self.get_channel_by_addr(addr)
        return OperationServiceStub(chan)  # type: ignore[no-untyped-call]

    def get_corresponding_operation_service_alpha(
        self,
        service_stub_class: type[ServiceStub],
    ) -> OperationServiceStubDeprecated:
        """Compatibility helper returning the alpha-version operations
        service stub for the same address as a generated service stub.

        See :meth:`get_corresponding_operation_service` for details. This
        method returns the older alpha operations stub for callers that need
        to interoperate with legacy server implementations.
        """

        addr = self.get_addr_from_stub(service_stub_class)
        chan = self.get_channel_by_addr(addr)
        return OperationServiceStubDeprecated(chan)  # type: ignore[no-untyped-call]

    def get_addr_from_stub(self, service_stub_class: type[ServiceStub]) -> str:
        """Resolve the concrete address for a generated service stub class.

        :param service_stub_class: The generated gRPC stub class for a
            service.
        :return: The resolved address string used by the SDK to reach that
            service (for example ``'host:port'`` or a resolver template
            expanded value).
        :rtype: str
        """

        service = from_stub_class(service_stub_class)
        return self.get_addr_from_service_name(service)

    def get_addr_from_service_name(self, service_name: str) -> str:
        """Resolve a logical service name into a transport address.

        The method strips a leading dot (``"."``) if present and delegates
        to the configured :class:`Resolver`.

        :param service_name: Logical service name as generated by stubs or
            conventions.
        :return: Resolved address string.
        :rtype: str
        """

        if len(service_name) > 1 and service_name[0] == ".":
            service_name = service_name[1:]
        return self._resolver.resolve(service_name)

    def get_addr_by_method(self, method_name: str) -> str:
        """Return the cached address for a fully-qualified RPC method name.

        If the method-to-service mapping has not been seen before it is
        computed using :func:`service_from_method_name`
        and resolved via :meth:`get_addr_from_service_name`. The result is
        cached to accelerate future lookups.

        :param method_name: Full RPC method string (``'/package.service/Method'``).
        :return: Resolved address string.
        :rtype: str
        """

        if method_name not in self._methods:
            service_name = service_from_method_name(method_name)
            self._methods[method_name] = self.get_addr_from_service_name(service_name)
        return self._methods[method_name]

    def get_channel_by_addr(self, addr: str) -> AddressChannel:
        """Request an :class:`AddressChannel` for the given resolved address.

        The method returns a pooled channel if available; otherwise a new
        underlying gRPC channel is created. Pooled channels with state
        :attr:`grpc.ChannelConnectivity.SHUTDOWN` are closed asynchronously and
        skipped.

        :param addr: Resolved address string.
        :return: An :class:`AddressChannel` wrapper for a gRPC channel.
        :rtype: :class:`AddressChannel`
        :raises ChannelClosedError: If the SDK channel has already been closed.
        """

        if self._closed:
            raise ChannelClosedError("Channel closed")
        if addr not in self._free_channels:
            self._free_channels[addr] = []
        chans = self._free_channels[addr]
        while len(chans) > 0:
            chan = chans.pop()
            if chan.get_state() != ChannelConnectivity.SHUTDOWN:
                return AddressChannel(chan, addr)
            self.bg_task(chan.close(None))

        return self.create_address_channel(addr)

    def return_channel(self, chan: AddressChannel | None) -> None:
        """Return an :class:`AddressChannel` to the internal pool.

        Channels returned to the pool will be reused by subsequent
        :meth:`get_channel_by_addr` calls up to the configured
        ``max_free_channels_per_address`` limit. Channels that are shut down
        or exceed the pool size are closed asynchronously.

        :param chan: The :class:`AddressChannel` to return, or ``None``.
        :raises ChannelClosedError: If the SDK channel has been closed.
        """

        if chan is None:
            return
        if self._closed:
            raise ChannelClosedError("Channel closed")
        if chan.address not in self._free_channels:
            self._free_channels[chan.address] = []
        if (
            chan.channel.get_state() != ChannelConnectivity.SHUTDOWN
            and len(self._free_channels[chan.address])
            < self._max_free_channels_per_address
        ):
            self._free_channels[chan.address].append(chan.channel)
        else:
            self.discard_channel(chan)

    def discard_channel(self, chan: AddressChannel | None) -> None:
        """Dispose of an :class:`AddressChannel` by scheduling its close.

        The close is performed asynchronously via :meth:`bg_task` to avoid
        blocking the caller.

        :param chan: The :class:`AddressChannel` to discard, or ``None``.
        :raises ChannelClosedError: If the SDK channel has been closed.
        """

        if chan is None:
            return
        if self._closed:
            raise ChannelClosedError("Channel closed")
        self.bg_task(chan.channel.close(None))

    def get_channel_by_method(self, method_name: str) -> AddressChannel:
        """Convenience to obtain an :class:`AddressChannel` for an RPC
        method name.
        The method resolves the address via :meth:`get_addr_by_method` and
        then calls :meth:`get_channel_by_addr` to obtain the channel.

        :param method_name: Full RPC method string.
        :return: An :class:`AddressChannel` bound to the resolved address.
        """

        addr = self.get_addr_by_method(method_name)
        return self.get_channel_by_addr(addr)

    def get_address_options(self, addr: str) -> ChannelArgumentType:
        """Compute effective gRPC channel options for a specific address.

        Global options are combined with per-address options and the SDK
        user-agent is appended via ``grpc.primary_user_agent``.

        :param addr: Resolved address string.
        :return: A sequence of channel option tuples ready to be passed to
            gRPC when creating a channel.
        :rtype: list of ``tuple[str, Any]``
        """

        ret = [opt for opt in self._global_options]
        if addr in self._address_options:
            ret.extend(self._address_options[addr])
        ret = set_user_agent_option(self.user_agent, ret)  # type: ignore[assignment]
        return ret

    def get_address_interceptors(self, addr: str) -> Sequence[ClientInterceptor]:
        """Return the ordered list of interceptors to apply to a channel.

        Global interceptors are applied first, then any per-address
        interceptors, and finally internal interceptors added by the
        channel implementation.

        :param addr: Resolved address string.
        :return: Combined global and per-address interceptors.
        :rtype: A sequence of :class:`ClientInterceptor`
        """

        ret = [opt for opt in self._global_interceptors]
        if addr in self._address_interceptors:
            ret.extend(self._address_interceptors[addr])
        ret.extend(self._global_interceptors_inner)
        return ret

    def create_address_channel(self, addr: str) -> AddressChannel:
        """Create a new underlying gRPC channel for the given address.

        The method composes options and interceptors, extracts known
        special options such as ``INSECURE`` and ``COMPRESSION``, and then
        constructs either a secure or insecure gRPC channel wrapper. The
        returned object is an :class:`AddressChannel` that pairs the gRPC
        channel with the resolved address string.

        :param addr: Resolved address string.
        :type addr: str
        :return: An :class:`AddressChannel` containing the created channel.
        :rtype: :class:`AddressChannel`
        """

        logger.debug(f"creating channel for {addr=}")
        opts = self.get_address_options(addr)
        opts, insecure = pop_option(opts, INSECURE, bool)
        opts, compression = pop_option(opts, COMPRESSION, Compression)
        interceptors = self.get_address_interceptors(addr)
        if insecure:
            return AddressChannel(
                insecure_channel(addr, opts, compression, interceptors),  # type: ignore[unused-ignore,no-any-return]
                addr,
            )
        else:
            return AddressChannel(
                secure_channel(  # type: ignore[unused-ignore,no-any-return]
                    addr,
                    self._tls_credentials,
                    opts,
                    compression,
                    interceptors,
                ),
                addr,
            )

    def unary_unary(  # type: ignore[unused-ignore,override]
        self,
        method_name: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
    ) -> UnaryUnaryMultiCallable[Req, Res]:  # type: ignore[unused-ignore,override]
        """
        A method to support using SDK channel as gRPC Channel.

        :param method_name:
            Full RPC method string, i.e., ``'/package.service/method'``.
        :type method_name: str
        :param request_serializer:
            A function that serializes a request message to bytes.
        :type request_serializer: SerializingFunction | None
        :param response_deserializer:
            A function that deserializes a response message from bytes.
        :type response_deserializer: DeserializingFunction | None
        :return:
            A :class:`UnaryUnaryMultiCallable` object that can be used to make
            the call.
        :rtype: :class:`NebiusUnaryUnaryMultiCallable` wrapper.
        """
        return NebiusUnaryUnaryMultiCallable(
            self,
            method_name,
            request_serializer,
            response_deserializer,
        )

    async def __aenter__(self) -> "Channel":
        """
        Enter the async context manager.
        Returns self to allow usage like::

            async with channel as chan:
                await chan.some_method()

        Will close the channel on exit.
        """
        return self

    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
        """
        Exit the async context manager.
        Calls close() to gracefully shut down resources.
        """
        await self.close(None)

    def get_state(self, try_to_connect: bool = False) -> ChannelConnectivity:
        """
        Nebius Python SDK channels are always ready unless closed.

        :param try_to_connect:
            Ignored parameter to satisfy the gRPC Channel interface.
        :type try_to_connect: bool
        :return:
            :attr:`grpc.ChannelConnectivity.READY` if the channel is open,
            :attr:`grpc.ChannelConnectivity.SHUTDOWN` if closed.
        :rtype: :class:`grpc.ChannelConnectivity`
        """
        if self._closed:
            return ChannelConnectivity.SHUTDOWN
        return ChannelConnectivity.READY

    async def wait_for_state_change(
        self,
        last_observed_state: ChannelConnectivity,
    ) -> None:
        """
        Nebius Python SDK channels are always ready unless closed.
        This method is provided to satisfy the gRPC Channel interface.

        :raises NotImplementedError:
        """
        raise NotImplementedError("this method has no meaning for this channel")

    async def channel_ready(self) -> None:
        """
        Channel is always ready, nothing to do here.
        """
        return

    def unary_stream(  # type: ignore[unused-ignore,override]
        self,
        method: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
    ) -> UnaryStreamMultiCallable[Req, Res]:  # type: ignore[unused-ignore]
        """
        Nebius Python SDK does not support streaming RPCs.

        :raises NotImplementedError:
        """
        raise NotImplementedError("Method not implemented")

    def stream_unary(  # type: ignore[unused-ignore,override]
        self,
        method: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
    ) -> StreamUnaryMultiCallable:
        """
        Nebius Python SDK does not support streaming RPCs.

        :raises NotImplementedError:
        """
        raise NotImplementedError("Method not implemented")

    def stream_stream(  # type: ignore[unused-ignore,override]
        self,
        method: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
    ) -> StreamStreamMultiCallable:
        """
        Nebius Python SDK does not support streaming RPCs.

        :raises NotImplementedError:
        """
        raise NotImplementedError("Method not implemented")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/vpc/v1/pool.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18nebius/vpc/v1/pool.proto\x12\rnebius.vpc.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a\x18nebius/annotations.proto\"\xa6\x01\n\x04Pool\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12+\n\x04spec\x18\x02 \x01(\x0b\x32\x17.nebius.vpc.v1.PoolSpecR\x04spec\x12\x31\n\x06status\x18\x03 \x01(\x0b\x32\x19.nebius.vpc.v1.PoolStatusR\x06status\"\xee\x01\n\x08PoolSpec\x12*\n\x0esource_pool_id\x18\x01 \x01(\tB\x04\xbaJ\x01\x02R\x0csourcePoolId\x12>\n\x07version\x18\x03 \x01(\x0e\x32\x18.nebius.vpc.v1.IpVersionB\n\xbaH\x03\xc8\x01\x01\xbaJ\x01\x02R\x07version\x12G\n\nvisibility\x18\x05 \x01(\x0e\x32\x1b.nebius.vpc.v1.IpVisibilityB\n\xbaH\x03\xc8\x01\x01\xbaJ\x01\x02R\nvisibility\x12-\n\x05\x63idrs\x18\x04 \x03(\x0b\x32\x17.nebius.vpc.v1.PoolCidrR\x05\x63idrs\"\x91\x03\n\x08PoolCidr\x12\x8e\x02\n\x04\x63idr\x18\x01 \x01(\tB\xf9\x01\xbaH\xf5\x01\xba\x01\x9d\x01\n\x11string.valid_cidr\x12.value must be a valid IP address, CIDR or mask\x1aXthis == \'\' || this.matches(\'^/([0-9]|[1-9][0-9]|1[0-2][0-8])$\') || this.isIpPrefix(true)\xba\x01N\n\x0fstring.ip_empty\x12/value is empty, which is not a valid IP address\x1a\nthis != \'\'\xc8\x01\x01R\x04\x63idr\x12<\n\x05state\x18\x02 \x01(\x0e\x32 .nebius.vpc.v1.AddressBlockStateB\x04\xbaJ\x01\x07R\x05state\x12\x36\n\x0fmax_mask_length\x18\x03 \x01(\x03\x42\x0e\xbaH\x07\"\x05\x18\x80\x01(\x00\xbaJ\x01\x07R\rmaxMaskLength\"\xfa\x01\n\nPoolStatus\x12\x35\n\x05state\x18\x01 \x01(\x0e\x32\x1f.nebius.vpc.v1.PoolStatus.StateR\x05state\x12\x14\n\x05\x63idrs\x18\x02 \x03(\tR\x05\x63idrs\x12\x19\n\x08scope_id\x18\x03 \x01(\tR\x07scopeId\x12=\n\nassignment\x18\x04 \x01(\x0b\x32\x1d.nebius.vpc.v1.PoolAssignmentR\nassignment\"E\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\x0c\n\x08\x43REATING\x10\x01\x12\t\n\x05READY\x10\x02\x12\x0c\n\x08\x44\x45LETING\x10\x03\"F\n\x0ePoolAssignment\x12\x1a\n\x08networks\x18\x01 \x03(\tR\x08networks\x12\x18\n\x07subnets\x18\x02 \x03(\tR\x07subnets*G\n\x11\x41\x64\x64ressBlockState\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\r\n\tAVAILABLE\x10\x01\x12\x0c\n\x08\x44ISABLED\x10\x02*;\n\tIpVersion\x12\x1a\n\x16IP_VERSION_UNSPECIFIED\x10\x00\x12\x08\n\x04IPV4\x10\x01\x12\x08\n\x04IPV6\x10\x02*F\n\x0cIpVisibility\x12\x1d\n\x19IP_VISIBILITY_UNSPECIFIED\x10\x00\x12\x0b\n\x07PRIVATE\x10\x01\x12\n\n\x06PUBLIC\x10\x02\x42P\n\x14\x61i.nebius.pub.vpc.v1B\tPoolProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/vpc/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.vpc.v1.pool_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.vpc.v1B\tPoolProtoP\001Z+github.com/nebius/gosdk/proto/nebius/vpc/v1'
  _POOLSPEC.fields_by_name['source_pool_id']._options = None
  _POOLSPEC.fields_by_name['source_pool_id']._serialized_options = b'\272J\001\002'
  _POOLSPEC.fields_by_name['version']._options = None
  _POOLSPEC.fields_by_name['version']._serialized_options = b'\272H\003\310\001\001\272J\001\002'
  _POOLSPEC.fields_by_name['visibility']._options = None
  _POOLSPEC.fields_by_name['visibility']._serialized_options = b'\272H\003\310\001\001\272J\001\002'
  _POOLCIDR.fields_by_name['cidr']._options = None
  _POOLCIDR.fields_by_name['cidr']._serialized_options = b'\272H\365\001\272\001\235\001\n\021string.valid_cidr\022.value must be a valid IP address, CIDR or mask\032Xthis == \'\' || this.matches(\'^/([0-9]|[1-9][0-9]|1[0-2][0-8])$\') || this.isIpPrefix(true)\272\001N\n\017string.ip_empty\022/value is empty, which is not a valid IP address\032\nthis != \'\'\310\001\001'
  _POOLCIDR.fields_by_name['state']._options = None
  _POOLCIDR.fields_by_name['state']._serialized_options = b'\272J\001\007'
  _POOLCIDR.fields_by_name['max_mask_length']._options = None
  _POOLCIDR.fields_by_name['max_mask_length']._serialized_options = b'\272H\007\"\005\030\200\001(\000\272J\001\007'
  _globals['_ADDRESSBLOCKSTATE']._serialized_start=1270
  _globals['_ADDRESSBLOCKSTATE']._serialized_end=1341
  _globals['_IPVERSION']._serialized_start=1343
  _globals['_IPVERSION']._serialized_end=1402
  _globals['_IPVISIBILITY']._serialized_start=1404
  _globals['_IPVISIBILITY']._serialized_end=1474
  _globals['_POOL']._serialized_start=132
  _globals['_POOL']._serialized_end=298
  _globals['_POOLSPEC']._serialized_start=301
  _globals['_POOLSPEC']._serialized_end=539
  _globals['_POOLCIDR']._serialized_start=542
  _globals['_POOLCIDR']._serialized_end=943
  _globals['_POOLSTATUS']._serialized_start=946
  _globals['_POOLSTATUS']._serialized_end=1196
  _globals['_POOLSTATUS_STATE']._serialized_start=1127
  _globals['_POOLSTATUS_STATE']._serialized_end=1196
  _globals['_POOLASSIGNMENT']._serialized_start=1198
  _globals['_POOLASSIGNMENT']._serialized_end=1268
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from nebius.api.nebius.compute.v1 import maintenance_event_pb2 as nebius_dot_compute_dot_v1_dot_maintenance__event__pb2
from nebius.api.nebius.compute.v1 import maintenance_service_pb2 as nebius_dot_compute_dot_v1_dot_maintenance__service__pb2


class MaintenanceServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetByInstance = channel.unary_unary(
                '/nebius.compute.v1.MaintenanceService/GetByInstance',
                request_serializer=nebius_dot_compute_dot_v1_dot_maintenance__service__pb2.GetMaintenanceEventByInstanceRequest.SerializeToString,
                response_deserializer=nebius_dot_compute_dot_v1_dot_maintenance__event__pb2.MaintenanceEvent.FromString,
                )
        self.ListActive = channel.unary_unary(
                '/nebius.compute.v1.MaintenanceService/ListActive',
                request_serializer=nebius_dot_compute_dot_v1_dot_maintenance__service__pb2.ListMaintenanceEventsRequest.SerializeToString,
                response_deserializer=nebius_dot_compute_dot_v1_dot_maintenance__service__pb2.ListMaintenanceEventsResponse.FromString,
                )


class MaintenanceServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetByInstance(self, request, context):
        """Returns only active maintenance event for the instance
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListActive(self, request, context):
        """Returns all unfinished maintenance events for the parent
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_MaintenanceServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetByInstance': grpc.unary_unary_rpc_method_handler(
                    servicer.GetByInstance,
                    request_deserializer=nebius_dot_compute_dot_v1_dot_maintenance__service__pb2.GetMaintenanceEventByInstanceRequest.FromString,
                    response_serializer=nebius_dot_compute_dot_v1_dot_maintenance__event__pb2.MaintenanceEvent.SerializeToString,
            ),
            'ListActive': grpc.unary_unary_rpc_method_handler(
                    servicer.ListActive,
                    request_deserializer=nebius_dot_compute_dot_v1_dot_maintenance__service__pb2.ListMaintenanceEventsRequest.FromString,
                    response_serializer=nebius_dot_compute_dot_v1_dot_maintenance__service__pb2.ListMaintenanceEventsResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'nebius.compute.v1.MaintenanceService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class MaintenanceService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetByInstance(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.compute.v1.MaintenanceService/GetByInstance',
            nebius_dot_compute_dot_v1_dot_maintenance__service__pb2.GetMaintenanceEventByInstanceRequest.SerializeToString,
            nebius_dot_compute_dot_v1_dot_maintenance__event__pb2.MaintenanceEvent.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListActive(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.compute.v1.MaintenanceService/ListActive',
            nebius_dot_compute_dot_v1_dot_maintenance__service__pb2.ListMaintenanceEventsRequest.SerializeToString,
            nebius_dot_compute_dot_v1_dot_maintenance__service__pb2.ListMaintenanceEventsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/msp/serverless/v1alpha1/endpoint.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.msp.v1alpha1 import cluster_pb2 as nebius_dot_msp_dot_v1alpha1_dot_cluster__pb2
from nebius.api.nebius.msp.v1alpha1.resource import template_pb2 as nebius_dot_msp_dot_v1alpha1_dot_resource_dot_template__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-nebius/msp/serverless/v1alpha1/endpoint.proto\x12\x1enebius.msp.serverless.v1alpha1\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a!nebius/msp/v1alpha1/cluster.proto\x1a+nebius/msp/v1alpha1/resource/template.proto\"\xea\x01\n\x08\x45ndpoint\x12\x46\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\x06\xbaH\x03\xc8\x01\x01R\x08metadata\x12H\n\x04spec\x18\x02 \x01(\x0b\x32,.nebius.msp.serverless.v1alpha1.EndpointSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec\x12L\n\x06status\x18\x03 \x01(\x0b\x32..nebius.msp.serverless.v1alpha1.EndpointStatusB\x04\xbaJ\x01\x05R\x06status\"\xa2\x02\n\x0c\x45ndpointSpec\x12 \n\x0b\x64\x65scription\x18\x01 \x01(\tR\x0b\x64\x65scription\x12)\n\nnetwork_id\x18\x02 \x01(\tB\n\xbaH\x03\xc8\x01\x01\xbaJ\x01\x02R\tnetworkId\x12)\n\x08username\x18\x03 \x01(\tB\r\xbaH\x03\xc8\x01\x01\xbaJ\x01\x04\xc0J\x01R\x08username\x12)\n\x08password\x18\x04 \x01(\tB\r\xbaH\x03\xc8\x01\x01\xbaJ\x01\x04\xc0J\x01R\x08password\x12\x1a\n\x04port\x18\x05 \x01(\rB\x06\xbaH\x03\xc8\x01\x01R\x04port\x12S\n\tcontainer\x18\x65 \x01(\x0b\x32\x35.nebius.msp.serverless.v1alpha1.EndpointContainerSpecR\tcontainer\"\xf4\x04\n\x15\x45ndpointContainerSpec\x12\x1c\n\x05image\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x05image\x12#\n\rreplica_count\x18\x02 \x01(\x03R\x0creplicaCount\x12X\n\x08template\x18\x03 \x01(\x0b\x32\x34.nebius.msp.serverless.v1alpha1.EndpointTemplateSpecB\x06\xbaH\x03\xc8\x01\x01R\x08template\x12\x18\n\x07\x63ommand\x18\x65 \x01(\tR\x07\x63ommand\x12\x12\n\x04\x61rgs\x18\x66 \x03(\tR\x04\x61rgs\x12S\n\x04\x65nvs\x18g \x03(\x0b\x32?.nebius.msp.serverless.v1alpha1.EndpointContainerSpec.EnvsEntryR\x04\x65nvs\x12x\n\x0esensitive_envs\x18h \x03(\x0b\x32H.nebius.msp.serverless.v1alpha1.EndpointContainerSpec.SensitiveEnvsEntryB\x07\xbaJ\x01\x04\xc0J\x01R\rsensitiveEnvs\x12\x46\n\x08liveness\x18\xc9\x01 \x01(\x0b\x32).nebius.msp.serverless.v1alpha1.ProbeSpecR\x08liveness\x1a\x37\n\tEnvsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x1a@\n\x12SensitiveEnvsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xca\x01\n\tProbeSpec\x12\x1a\n\x04path\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x04path\x12#\n\rdelay_seconds\x18\x05 \x01(\x03R\x0c\x64\x65laySeconds\x12\x31\n\x0eperiod_seconds\x18\x06 \x01(\x03\x42\n\xbaH\x07\xc8\x01\x01\"\x02 \x00R\rperiodSeconds\x12\x37\n\x11\x66\x61ilure_threshold\x18\x07 \x01(\x03\x42\n\xbaH\x07\xc8\x01\x01\"\x02 \x00R\x10\x66\x61ilureThresholdJ\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05\"i\n\x14\x45ndpointTemplateSpec\x12Q\n\tresources\x18\x01 \x01(\x0b\x32+.nebius.msp.v1alpha1.resource.ResourcesSpecB\x06\xbaH\x03\xc8\x01\x01R\tresources\"\xb9\x01\n\x0e\x45ndpointStatus\x12>\n\x05phase\x18\x01 \x01(\x0e\x32(.nebius.msp.v1alpha1.ClusterStatus.PhaseR\x05phase\x12>\n\x05state\x18\x02 \x01(\x0e\x32(.nebius.msp.v1alpha1.ClusterStatus.StateR\x05state\x12\'\n\x0fpublic_endpoint\x18\x03 \x01(\tR\x0epublicEndpointBv\n%ai.nebius.pub.msp.serverless.v1alpha1B\rEndpointProtoP\x01Z<github.com/nebius/gosdk/proto/nebius/msp/serverless/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.msp.serverless.v1alpha1.endpoint_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n%ai.nebius.pub.msp.serverless.v1alpha1B\rEndpointProtoP\001Z<github.com/nebius/gosdk/proto/nebius/msp/serverless/v1alpha1'
  _ENDPOINT.fields_by_name['metadata']._options = None
  _ENDPOINT.fields_by_name['metadata']._serialized_options = b'\272H\003\310\001\001'
  _ENDPOINT.fields_by_name['spec']._options = None
  _ENDPOINT.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _ENDPOINT.fields_by_name['status']._options = None
  _ENDPOINT.fields_by_name['status']._serialized_options = b'\272J\001\005'
  _ENDPOINTSPEC.fields_by_name['network_id']._options = None
  _ENDPOINTSPEC.fields_by_name['network_id']._serialized_options = b'\272H\003\310\001\001\272J\001\002'
  _ENDPOINTSPEC.fields_by_name['username']._options = None
  _ENDPOINTSPEC.fields_by_name['username']._serialized_options = b'\272H\003\310\001\001\272J\001\004\300J\001'
  _ENDPOINTSPEC.fields_by_name['password']._options = None
  _ENDPOINTSPEC.fields_by_name['password']._serialized_options = b'\272H\003\310\001\001\272J\001\004\300J\001'
  _ENDPOINTSPEC.fields_by_name['port']._options = None
  _ENDPOINTSPEC.fields_by_name['port']._serialized_options = b'\272H\003\310\001\001'
  _ENDPOINTCONTAINERSPEC_ENVSENTRY._options = None
  _ENDPOINTCONTAINERSPEC_ENVSENTRY._serialized_options = b'8\001'
  _ENDPOINTCONTAINERSPEC_SENSITIVEENVSENTRY._options = None
  _ENDPOINTCONTAINERSPEC_SENSITIVEENVSENTRY._serialized_options = b'8\001'
  _ENDPOINTCONTAINERSPEC.fields_by_name['image']._options = None
  _ENDPOINTCONTAINERSPEC.fields_by_name['image']._serialized_options = b'\272H\003\310\001\001'
  _ENDPOINTCONTAINERSPEC.fields_by_name['template']._options = None
  _ENDPOINTCONTAINERSPEC.fields_by_name['template']._serialized_options = b'\272H\003\310\001\001'
  _ENDPOINTCONTAINERSPEC.fields_by_name['sensitive_envs']._options = None
  _ENDPOINTCONTAINERSPEC.fields_by_name['sensitive_envs']._serialized_options = b'\272J\001\004\300J\001'
  _PROBESPEC.fields_by_name['path']._options = None
  _PROBESPEC.fields_by_name['path']._serialized_options = b'\272H\003\310\001\001'
  _PROBESPEC.fields_by_name['period_seconds']._options = None
  _PROBESPEC.fields_by_name['period_seconds']._serialized_options = b'\272H\007\310\001\001\"\002 \000'
  _PROBESPEC.fields_by_name['failure_threshold']._options = None
  _PROBESPEC.fields_by_name['failure_threshold']._serialized_options = b'\272H\007\310\001\001\"\002 \000'
  _ENDPOINTTEMPLATESPEC.fields_by_name['resources']._options = None
  _ENDPOINTTEMPLATESPEC.fields_by_name['resources']._serialized_options = b'\272H\003\310\001\001'
  _globals['_ENDPOINT']._serialized_start=250
  _globals['_ENDPOINT']._serialized_end=484
  _globals['_ENDPOINTSPEC']._serialized_start=487
  _globals['_ENDPOINTSPEC']._serialized_end=777
  _globals['_ENDPOINTCONTAINERSPEC']._serialized_start=780
  _globals['_ENDPOINTCONTAINERSPEC']._serialized_end=1408
  _globals['_ENDPOINTCONTAINERSPEC_ENVSENTRY']._serialized_start=1287
  _globals['_ENDPOINTCONTAINERSPEC_ENVSENTRY']._serialized_end=1342
  _globals['_ENDPOINTCONTAINERSPEC_SENSITIVEENVSENTRY']._serialized_start=1344
  _globals['_ENDPOINTCONTAINERSPEC_SENSITIVEENVSENTRY']._serialized_end=1408
  _globals['_PROBESPEC']._serialized_start=1411
  _globals['_PROBESPEC']._serialized_end=1613
  _globals['_ENDPOINTTEMPLATESPEC']._serialized_start=1615
  _globals['_ENDPOINTTEMPLATESPEC']._serialized_end=1720
  _globals['_ENDPOINTSTATUS']._serialized_start=1723
  _globals['_ENDPOINTSTATUS']._serialized_end=1908
# @@protoc_insertion_point(module_scope)



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class SuspensionState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    SUSPENSION_STATE_UNSPECIFIED: _ClassVar[SuspensionState]
    NONE: _ClassVar[SuspensionState]
    SUSPENDING: _ClassVar[SuspensionState]
    SUSPENDED: _ClassVar[SuspensionState]
    RESUMING: _ClassVar[SuspensionState]
SUSPENSION_STATE_UNSPECIFIED: SuspensionState
NONE: SuspensionState
SUSPENDING: SuspensionState
SUSPENDED: SuspensionState
RESUMING: SuspensionState



from nebius.api.nebius.storage.v1 import base_pb2 as _base_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class LifecycleConfiguration(_message.Message):
    __slots__ = ["rules"]
    RULES_FIELD_NUMBER: _ClassVar[int]
    rules: _containers.RepeatedCompositeFieldContainer[LifecycleRule]
    def __init__(self, rules: _Optional[_Iterable[_Union[LifecycleRule, _Mapping]]] = ...) -> None: ...

class LifecycleRule(_message.Message):
    __slots__ = ["id", "status", "filter", "expiration", "noncurrent_version_expiration", "abort_incomplete_multipart_upload", "transition", "noncurrent_version_transition"]
    class Status(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        STATUS_UNSPECIFIED: _ClassVar[LifecycleRule.Status]
        ENABLED: _ClassVar[LifecycleRule.Status]
        DISABLED: _ClassVar[LifecycleRule.Status]
    STATUS_UNSPECIFIED: LifecycleRule.Status
    ENABLED: LifecycleRule.Status
    DISABLED: LifecycleRule.Status
    ID_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    EXPIRATION_FIELD_NUMBER: _ClassVar[int]
    NONCURRENT_VERSION_EXPIRATION_FIELD_NUMBER: _ClassVar[int]
    ABORT_INCOMPLETE_MULTIPART_UPLOAD_FIELD_NUMBER: _ClassVar[int]
    TRANSITION_FIELD_NUMBER: _ClassVar[int]
    NONCURRENT_VERSION_TRANSITION_FIELD_NUMBER: _ClassVar[int]
    id: str
    status: LifecycleRule.Status
    filter: LifecycleFilter
    expiration: LifecycleExpiration
    noncurrent_version_expiration: LifecycleNoncurrentVersionExpiration
    abort_incomplete_multipart_upload: LifecycleAbortIncompleteMultipartUpload
    transition: LifecycleTransition
    noncurrent_version_transition: LifecycleNoncurrentVersionTransition
    def __init__(self, id: _Optional[str] = ..., status: _Optional[_Union[LifecycleRule.Status, str]] = ..., filter: _Optional[_Union[LifecycleFilter, _Mapping]] = ..., expiration: _Optional[_Union[LifecycleExpiration, _Mapping]] = ..., noncurrent_version_expiration: _Optional[_Union[LifecycleNoncurrentVersionExpiration, _Mapping]] = ..., abort_incomplete_multipart_upload: _Optional[_Union[LifecycleAbortIncompleteMultipartUpload, _Mapping]] = ..., transition: _Optional[_Union[LifecycleTransition, _Mapping]] = ..., noncurrent_version_transition: _Optional[_Union[LifecycleNoncurrentVersionTransition, _Mapping]] = ...) -> None: ...

class LifecycleFilter(_message.Message):
    __slots__ = ["prefix", "object_size_greater_than_bytes", "object_size_less_than_bytes"]
    PREFIX_FIELD_NUMBER: _ClassVar[int]
    OBJECT_SIZE_GREATER_THAN_BYTES_FIELD_NUMBER: _ClassVar[int]
    OBJECT_SIZE_LESS_THAN_BYTES_FIELD_NUMBER: _ClassVar[int]
    prefix: str
    object_size_greater_than_bytes: int
    object_size_less_than_bytes: int
    def __init__(self, prefix: _Optional[str] = ..., object_size_greater_than_bytes: _Optional[int] = ..., object_size_less_than_bytes: _Optional[int] = ...) -> None: ...

class LifecycleExpiration(_message.Message):
    __slots__ = ["date", "days", "expired_object_delete_marker"]
    DATE_FIELD_NUMBER: _ClassVar[int]
    DAYS_FIELD_NUMBER: _ClassVar[int]
    EXPIRED_OBJECT_DELETE_MARKER_FIELD_NUMBER: _ClassVar[int]
    date: _timestamp_pb2.Timestamp
    days: int
    expired_object_delete_marker: bool
    def __init__(self, date: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., days: _Optional[int] = ..., expired_object_delete_marker: bool = ...) -> None: ...

class LifecycleNoncurrentVersionExpiration(_message.Message):
    __slots__ = ["newer_noncurrent_versions", "noncurrent_days"]
    NEWER_NONCURRENT_VERSIONS_FIELD_NUMBER: _ClassVar[int]
    NONCURRENT_DAYS_FIELD_NUMBER: _ClassVar[int]
    newer_noncurrent_versions: int
    noncurrent_days: int
    def __init__(self, newer_noncurrent_versions: _Optional[int] = ..., noncurrent_days: _Optional[int] = ...) -> None: ...

class LifecycleAbortIncompleteMultipartUpload(_message.Message):
    __slots__ = ["days_after_initiation"]
    DAYS_AFTER_INITIATION_FIELD_NUMBER: _ClassVar[int]
    days_after_initiation: int
    def __init__(self, days_after_initiation: _Optional[int] = ...) -> None: ...

class LifecycleTransition(_message.Message):
    __slots__ = ["date", "days", "storage_class"]
    DATE_FIELD_NUMBER: _ClassVar[int]
    DAYS_FIELD_NUMBER: _ClassVar[int]
    STORAGE_CLASS_FIELD_NUMBER: _ClassVar[int]
    date: _timestamp_pb2.Timestamp
    days: int
    storage_class: _base_pb2.StorageClass
    def __init__(self, date: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., days: _Optional[int] = ..., storage_class: _Optional[_Union[_base_pb2.StorageClass, str]] = ...) -> None: ...

class LifecycleNoncurrentVersionTransition(_message.Message):
    __slots__ = ["newer_noncurrent_versions", "noncurrent_days", "storage_class"]
    NEWER_NONCURRENT_VERSIONS_FIELD_NUMBER: _ClassVar[int]
    NONCURRENT_DAYS_FIELD_NUMBER: _ClassVar[int]
    STORAGE_CLASS_FIELD_NUMBER: _ClassVar[int]
    newer_noncurrent_versions: int
    noncurrent_days: int
    storage_class: _base_pb2.StorageClass
    def __init__(self, newer_noncurrent_versions: _Optional[int] = ..., noncurrent_days: _Optional[int] = ..., storage_class: _Optional[_Union[_base_pb2.StorageClass, str]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v1/access_permit_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius.iam.v1 import access_permit_pb2 as nebius_dot_iam_dot_v1_dot_access__permit__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n)nebius/iam/v1/access_permit_service.proto\x12\rnebius.iam.v1\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a nebius/common/v1/operation.proto\x1a!nebius/iam/v1/access_permit.proto\"\x90\x01\n\x19\x43reateAccessPermitRequest\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12\x33\n\x04spec\x18\x02 \x01(\x0b\x32\x1f.nebius.iam.v1.AccessPermitSpecR\x04spec\"\x8a\x01\n\x17ListAccessPermitRequest\x12\x1b\n\tparent_id\x18\x01 \x01(\tR\x08parentId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\x12\x16\n\x06\x66ilter\x18\x04 \x01(\tR\x06\x66ilter\"+\n\x19\x44\x65leteAccessPermitRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"(\n\x16GetAccessPermitRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"u\n\x18ListAccessPermitResponse\x12\x31\n\x05items\x18\x01 \x03(\x0b\x32\x1b.nebius.iam.v1.AccessPermitR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\xe7\x02\n\x13\x41\x63\x63\x65ssPermitService\x12O\n\x06\x43reate\x12(.nebius.iam.v1.CreateAccessPermitRequest\x1a\x1b.nebius.common.v1.Operation\x12W\n\x04List\x12&.nebius.iam.v1.ListAccessPermitRequest\x1a\'.nebius.iam.v1.ListAccessPermitResponse\x12O\n\x06\x44\x65lete\x12(.nebius.iam.v1.DeleteAccessPermitRequest\x1a\x1b.nebius.common.v1.Operation\x12I\n\x03Get\x12%.nebius.iam.v1.GetAccessPermitRequest\x1a\x1b.nebius.iam.v1.AccessPermit\x1a\n\xbaJ\x07\x63pl.iamB_\n\x14\x61i.nebius.pub.iam.v1B\x18\x41\x63\x63\x65ssPermitServiceProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v1.access_permit_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v1B\030AccessPermitServiceProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v1'
  _ACCESSPERMITSERVICE._options = None
  _ACCESSPERMITSERVICE._serialized_options = b'\272J\007cpl.iam'
  _globals['_CREATEACCESSPERMITREQUEST']._serialized_start=189
  _globals['_CREATEACCESSPERMITREQUEST']._serialized_end=333
  _globals['_LISTACCESSPERMITREQUEST']._serialized_start=336
  _globals['_LISTACCESSPERMITREQUEST']._serialized_end=474
  _globals['_DELETEACCESSPERMITREQUEST']._serialized_start=476
  _globals['_DELETEACCESSPERMITREQUEST']._serialized_end=519
  _globals['_GETACCESSPERMITREQUEST']._serialized_start=521
  _globals['_GETACCESSPERMITREQUEST']._serialized_end=561
  _globals['_LISTACCESSPERMITRESPONSE']._serialized_start=563
  _globals['_LISTACCESSPERMITRESPONSE']._serialized_end=680
  _globals['_ACCESSPERMITSERVICE']._serialized_start=683
  _globals['_ACCESSPERMITSERVICE']._serialized_end=1042
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/mysterybox/v1/secret.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.mysterybox.v1 import secret_version_pb2 as nebius_dot_mysterybox_dot_v1_dot_secret__version__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!nebius/mysterybox/v1/secret.proto\x12\x14nebius.mysterybox.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a\x18nebius/annotations.proto\x1a)nebius/mysterybox/v1/secret_version.proto\"\xc0\x01\n\x06Secret\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12\x34\n\x04spec\x18\x02 \x01(\x0b\x32 .nebius.mysterybox.v1.SecretSpecR\x04spec\x12@\n\x06status\x18\x03 \x01(\x0b\x32\".nebius.mysterybox.v1.SecretStatusB\x04\xbaJ\x01\x05R\x06status\"\xd5\x01\n\nSecretSpec\x12 \n\x0b\x64\x65scription\x18\x01 \x01(\tR\x0b\x64\x65scription\x12\x37\n\x12primary_version_id\x18\x03 \x01(\tB\x04\xbaJ\x01\x07H\x00R\x10primaryVersionId\x88\x01\x01\x12U\n\x0esecret_version\x18\x04 \x01(\x0b\x32\'.nebius.mysterybox.v1.SecretVersionSpecB\x05\xbaJ\x02\x04\x02R\rsecretVersionB\x15\n\x13_primary_version_id\"\xb9\x02\n\x0cSecretStatus\x12>\n\x05state\x18\x01 \x01(\x0e\x32(.nebius.mysterybox.v1.SecretStatus.StateR\x05state\x12\x39\n\ndeleted_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tdeletedAt\x12\x35\n\x08purge_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x07purgeAt\x12/\n\x14\x65\x66\x66\x65\x63tive_kms_key_id\x18\x04 \x01(\tR\x11\x65\x66\x66\x65\x63tiveKmsKeyId\"F\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\n\n\x06\x41\x43TIVE\x10\x01\x12\x1a\n\x16SCHEDULED_FOR_DELETION\x10\x02\x42`\n\x1b\x61i.nebius.pub.mysterybox.v1B\x0bSecretProtoP\x01Z2github.com/nebius/gosdk/proto/nebius/mysterybox/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.mysterybox.v1.secret_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\033ai.nebius.pub.mysterybox.v1B\013SecretProtoP\001Z2github.com/nebius/gosdk/proto/nebius/mysterybox/v1'
  _SECRET.fields_by_name['status']._options = None
  _SECRET.fields_by_name['status']._serialized_options = b'\272J\001\005'
  _SECRETSPEC.fields_by_name['primary_version_id']._options = None
  _SECRETSPEC.fields_by_name['primary_version_id']._serialized_options = b'\272J\001\007'
  _SECRETSPEC.fields_by_name['secret_version']._options = None
  _SECRETSPEC.fields_by_name['secret_version']._serialized_options = b'\272J\002\004\002'
  _globals['_SECRET']._serialized_start=195
  _globals['_SECRET']._serialized_end=387
  _globals['_SECRETSPEC']._serialized_start=390
  _globals['_SECRETSPEC']._serialized_end=603
  _globals['_SECRETSTATUS']._serialized_start=606
  _globals['_SECRETSTATUS']._serialized_end=919
  _globals['_SECRETSTATUS_STATE']._serialized_start=849
  _globals['_SECRETSTATUS_STATE']._serialized_end=919
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/registry/v1/registry.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!nebius/registry/v1/registry.proto\x12\x12nebius.registry.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\"\xd2\x01\n\x08Registry\x12\x46\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\x06\xbaH\x03\xc8\x01\x01R\x08metadata\x12<\n\x04spec\x18\x02 \x01(\x0b\x32 .nebius.registry.v1.RegistrySpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec\x12@\n\x06status\x18\x03 \x01(\x0b\x32\".nebius.registry.v1.RegistryStatusB\x04\xbaJ\x01\x05R\x06status\"Y\n\x0cRegistrySpec\x12 \n\x0b\x64\x65scription\x18\x01 \x01(\tR\x0b\x64\x65scription\x12\'\n\x0cimages_count\x18\x02 \x01(\x05\x42\x04\xbaJ\x01\x05R\x0bimagesCount\"\xd8\x01\n\x0eRegistryStatus\x12>\n\x05state\x18\n \x01(\x0e\x32(.nebius.registry.v1.RegistryStatus.StateR\x05state\x12!\n\x0cimages_count\x18\x0b \x01(\x05R\x0bimagesCount\x12#\n\rregistry_fqdn\x18\x0f \x01(\tR\x0cregistryFqdn\">\n\x05State\x12\x0c\n\x08\x43REATING\x10\x00\x12\n\n\x06\x41\x43TIVE\x10\x01\x12\x0c\n\x08\x44\x45LETING\x10\x02\x12\r\n\tSUSPENDED\x10\x03\x42^\n\x19\x61i.nebius.pub.registry.v1B\rRegistryProtoP\x01Z0github.com/nebius/gosdk/proto/nebius/registry/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.registry.v1.registry_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031ai.nebius.pub.registry.v1B\rRegistryProtoP\001Z0github.com/nebius/gosdk/proto/nebius/registry/v1'
  _REGISTRY.fields_by_name['metadata']._options = None
  _REGISTRY.fields_by_name['metadata']._serialized_options = b'\272H\003\310\001\001'
  _REGISTRY.fields_by_name['spec']._options = None
  _REGISTRY.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _REGISTRY.fields_by_name['status']._options = None
  _REGISTRY.fields_by_name['status']._serialized_options = b'\272J\001\005'
  _REGISTRYSPEC.fields_by_name['images_count']._options = None
  _REGISTRYSPEC.fields_by_name['images_count']._serialized_options = b'\272J\001\005'
  _globals['_REGISTRY']._serialized_start=146
  _globals['_REGISTRY']._serialized_end=356
  _globals['_REGISTRYSPEC']._serialized_start=358
  _globals['_REGISTRYSPEC']._serialized_end=447
  _globals['_REGISTRYSTATUS']._serialized_start=450
  _globals['_REGISTRYSTATUS']._serialized_end=666
  _globals['_REGISTRYSTATUS_STATE']._serialized_start=604
  _globals['_REGISTRYSTATUS_STATE']._serialized_end=666
# @@protoc_insertion_point(module_scope)



import re
from collections.abc import Generator
from dataclasses import dataclass
from enum import Enum, auto
from json import dumps, loads

from .fieldmask import Error, FieldKey, Mask

simple_string_token_set = "[a-zA-Z0-9_]"  # noqa: S105 # not a password
simple_string_token_start = re.compile("^" + simple_string_token_set)
simple_string_token = re.compile("^(" + simple_string_token_set + "+)")
space_chars = " \r\n\t"
space_set = frozenset(space_chars)
max_context = 30  # maximum length of context to show in error messages.
context_back = 12  # number of characters to show before the error position.
error_mark = "\u20de"  # combining enclosing square


def _possible_ellipsis(s: str, max_context: int) -> str:
    if len(s) > max_context:
        return s[: max_context - 3] + "..."
    return s


def _context_around(s: str, pos: int) -> str:
    ctx_start = pos
    if ctx_start >= context_back:
        ctx_start -= context_back
    else:
        ctx_start = 0

    delta = pos - ctx_start
    ctx_str = _possible_ellipsis(s[ctx_start:], max_context)
    with_mark = ctx_str[:delta] + error_mark + ctx_str[delta:]
    return f"at position {pos} near {dumps(with_mark, ensure_ascii=False)}"


class ParseError(Error):
    pass


class ContextedParseError(ParseError):
    def __init__(self, source: str, position: int, summary: str) -> None:
        self.input = source
        self.position = position
        self.summary = summary

        super().__init__(f"{summary} {_context_around(source, position)}")


class TokenType(Enum):
    COMMA = auto()
    DOT = auto()
    LBRACE = auto()
    RBRACE = auto()
    WILD_CARD = auto()
    PLAIN_KEY = auto()
    QUOTED_KEY = auto()
    EOL = auto()


@dataclass
class Token:
    type: TokenType
    value: str
    pos: int

    def copy(self) -> "Token":
        return Token(
            type=self.type,
            value=self.value,
            pos=self.pos,
        )

    def unquote(self) -> "Token":
        if self.type != TokenType.QUOTED_KEY:
            return self.copy()
        unquoted = loads(self.value)
        if not isinstance(unquoted, str):
            raise ParseError(f"Token is not a quoted string: {self!r}")
        return Token(
            type=TokenType.PLAIN_KEY,
            value=unquoted,
            pos=self.pos,
        )

    def __repr__(self) -> str:
        if self.type != TokenType.QUOTED_KEY:
            val = dumps(self.value)
        else:
            val = self.value
        return f"Token{self.type.name}({val} pos {self.pos})"


class Lexer:
    def __init__(self, source: str) -> None:
        self.source = source
        self.pos: int = 0

    def next_token(self) -> Token:
        while self.pos < len(self.source) and self.source[self.pos] in space_set:
            self.pos += 1

        if self.pos >= len(self.source):
            return Token(
                type=TokenType.EOL,
                value="",
                pos=self.pos,
            )
        start = self.pos
        if self.starts_with(","):
            self.consume(",")
            return Token(
                type=TokenType.COMMA,
                value=",",
                pos=start,
            )
        elif self.starts_with("."):
            self.consume(".")
            return Token(
                type=TokenType.DOT,
                value=".",
                pos=start,
            )
        elif self.starts_with("("):
            self.consume("(")
            return Token(
                type=TokenType.LBRACE,
                value="(",
                pos=start,
            )
        elif self.starts_with(")"):
            self.consume(")")
            return Token(
                type=TokenType.RBRACE,
                value=")",
                pos=start,
            )
        elif self.starts_with("*"):
            self.consume("*")
            return Token(
                type=TokenType.WILD_CARD,
                value="*",
                pos=start,
            )
        elif self.starts_with('"'):
            return self.scan_quoted_key()
        elif simple_string_token_start.match(self.source[self.pos :]) is not None:
            return self.scan_plain_key()
        else:
            raise ContextedParseError(self.source, self.pos, "unexpected symbol")

    def scan_plain_key(self) -> Token:
        start = self.pos
        re_match = simple_string_token.match(self.source[start:])
        if re_match is None:
            raise ContextedParseError(
                self.source, self.pos, "unexpected match mismatch"
            )
        self.consume(re_match[0])
        return Token(
            type=TokenType.PLAIN_KEY,
            value=re_match[0],
            pos=start,
        )

    def scan_quoted_key(self) -> Token:
        start = self.pos
        self.consume('"')
        while self.pos < len(self.source) and not self.starts_with('"'):
            if self.starts_with("\\"):
                self.pos += 2
            else:
                self.pos += 1
        if self.starts_with('"'):
            self.consume('"')
            return Token(
                type=TokenType.QUOTED_KEY,
                value=self.source[start : self.pos],
                pos=start,
            )
        raise ContextedParseError(self.source, start, "unterminated quoted string")

    def consume(self, prefix: str) -> None:
        self.pos += len(prefix)

    def starts_with(self, prefix: str) -> bool:
        return self.source[self.pos :].startswith(prefix)

    def __iter__(self) -> Generator[Token, None, None]:
        while True:
            tok = self.next_token()
            try:
                tok = tok.unquote()
            except Exception as e:
                raise ContextedParseError(self.source, tok.pos, f"{e}") from e
            if tok.type == TokenType.EOL:
                return
            yield tok


class Level:
    def __init__(self) -> None:
        root = Mask()
        self.starts = list[Mask]([root])
        self.ends = list[Mask]()
        self.active = list[Mask]([root])
        self.prev: Level | None = None
        self.pos: int = 0

    def add_key(self, k: FieldKey) -> None:
        mask: Mask | None = None
        new_active = list[Mask]()
        for a in self.active:
            if k in a.field_parts:
                new_active.append(a.field_parts[k])
            else:
                if mask is None:
                    mask = Mask()
                    new_active.append(mask)
                a.field_parts[k] = mask
        self.active = new_active

    def add_any(self) -> None:
        mask: Mask | None = None
        new_active = list[Mask]()
        for a in self.active:
            if a.any is not None:
                new_active.append(a.any)
            else:
                if mask is None:
                    mask = Mask()
                    new_active.append(mask)
                a.any = mask
        self.active = new_active

    def new_mask(self) -> None:
        self.ends.extend(self.active)
        self.active = self.starts

    def push_level(self, pos: int) -> "Level":
        nl = Level()
        nl.pos = pos
        nl.prev = self
        nl.starts = self.active
        nl.active = self.active
        return nl

    def pop_level(self) -> "Level|None":
        p = self.prev
        if p is not None:
            p.active = self.ends
            p.active.extend(self.active)
        return p


class State(Enum):
    KEY = auto()
    SEPARATOR = auto()
    LEVEL_START = auto()


def parse(source: str) -> Mask:
    if not isinstance(source, str):  # type: ignore[unused-ignore]
        raise ParseError(f"wrong type of source: {type(source)}, expected str")
    if len(source.lstrip(space_chars)) == 0:
        return Mask()
    tokens = [t for t in Lexer(source)]
    lvl = Level()
    root = lvl.starts[0]
    pos = 0
    state: State = State.KEY

    while True:
        if pos >= len(tokens):
            break
        tok = tokens[pos]
        match state:
            case State.LEVEL_START:
                if tok.type == TokenType.RBRACE:
                    state = State.SEPARATOR
                else:
                    state = State.KEY
                continue
            case State.KEY:
                match tok.type:
                    case TokenType.PLAIN_KEY:
                        lvl.add_key(FieldKey(tok.value))
                        state = State.SEPARATOR
                    case TokenType.WILD_CARD:
                        lvl.add_any()
                        state = State.SEPARATOR
                    case TokenType.LBRACE:
                        lvl = lvl.push_level(tok.pos)
                        state = State.LEVEL_START
                    case _:
                        raise ContextedParseError(
                            source,
                            tok.pos,
                            f"unexpected token {tok}, expecting field or submask",
                        )
            case State.SEPARATOR:
                match tok.type:
                    case TokenType.DOT:
                        state = State.KEY
                    case TokenType.COMMA:
                        lvl.new_mask()
                        state = State.KEY
                    case TokenType.RBRACE:
                        lvl_ = lvl.pop_level()
                        if lvl_ is None:
                            raise ContextedParseError(
                                source, tok.pos, "unmatched right brace"
                            )
                        lvl = lvl_
                        state = State.SEPARATOR
                    case _:
                        raise ContextedParseError(
                            source,
                            tok.pos,
                            f"unexpected token {tok}, expecting separator or closing"
                            " brace",
                        )
            case _:  # type: ignore[unused-ignore]
                raise ParseError(f"state machine corruption: unknown state {state}")
        pos += 1
    if lvl.prev is not None:
        raise ContextedParseError(
            source,
            lvl.pos,
            "unclosed left brace",
        )
    if state != State.SEPARATOR:
        raise ParseError("unexpected end of mask")
    return root.copy()



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.compute.v1 import network_interface_pb2 as _network_interface_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class InstanceRecoveryPolicy(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    RECOVER: _ClassVar[InstanceRecoveryPolicy]
    FAIL: _ClassVar[InstanceRecoveryPolicy]
RECOVER: InstanceRecoveryPolicy
FAIL: InstanceRecoveryPolicy

class Instance(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: InstanceSpec
    status: InstanceStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[InstanceSpec, _Mapping]] = ..., status: _Optional[_Union[InstanceStatus, _Mapping]] = ...) -> None: ...

class InstanceSpec(_message.Message):
    __slots__ = ["service_account_id", "resources", "gpu_cluster", "network_interfaces", "boot_disk", "secondary_disks", "filesystems", "cloud_init_user_data", "stopped", "recovery_policy", "preemptible", "hostname"]
    SERVICE_ACCOUNT_ID_FIELD_NUMBER: _ClassVar[int]
    RESOURCES_FIELD_NUMBER: _ClassVar[int]
    GPU_CLUSTER_FIELD_NUMBER: _ClassVar[int]
    NETWORK_INTERFACES_FIELD_NUMBER: _ClassVar[int]
    BOOT_DISK_FIELD_NUMBER: _ClassVar[int]
    SECONDARY_DISKS_FIELD_NUMBER: _ClassVar[int]
    FILESYSTEMS_FIELD_NUMBER: _ClassVar[int]
    CLOUD_INIT_USER_DATA_FIELD_NUMBER: _ClassVar[int]
    STOPPED_FIELD_NUMBER: _ClassVar[int]
    RECOVERY_POLICY_FIELD_NUMBER: _ClassVar[int]
    PREEMPTIBLE_FIELD_NUMBER: _ClassVar[int]
    HOSTNAME_FIELD_NUMBER: _ClassVar[int]
    service_account_id: str
    resources: ResourcesSpec
    gpu_cluster: InstanceGpuClusterSpec
    network_interfaces: _containers.RepeatedCompositeFieldContainer[_network_interface_pb2.NetworkInterfaceSpec]
    boot_disk: AttachedDiskSpec
    secondary_disks: _containers.RepeatedCompositeFieldContainer[AttachedDiskSpec]
    filesystems: _containers.RepeatedCompositeFieldContainer[AttachedFilesystemSpec]
    cloud_init_user_data: str
    stopped: bool
    recovery_policy: InstanceRecoveryPolicy
    preemptible: PreemptibleSpec
    hostname: str
    def __init__(self, service_account_id: _Optional[str] = ..., resources: _Optional[_Union[ResourcesSpec, _Mapping]] = ..., gpu_cluster: _Optional[_Union[InstanceGpuClusterSpec, _Mapping]] = ..., network_interfaces: _Optional[_Iterable[_Union[_network_interface_pb2.NetworkInterfaceSpec, _Mapping]]] = ..., boot_disk: _Optional[_Union[AttachedDiskSpec, _Mapping]] = ..., secondary_disks: _Optional[_Iterable[_Union[AttachedDiskSpec, _Mapping]]] = ..., filesystems: _Optional[_Iterable[_Union[AttachedFilesystemSpec, _Mapping]]] = ..., cloud_init_user_data: _Optional[str] = ..., stopped: bool = ..., recovery_policy: _Optional[_Union[InstanceRecoveryPolicy, str]] = ..., preemptible: _Optional[_Union[PreemptibleSpec, _Mapping]] = ..., hostname: _Optional[str] = ...) -> None: ...

class PreemptibleSpec(_message.Message):
    __slots__ = ["on_preemption", "priority"]
    class PreemptionPolicy(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        UNSPECIFIED: _ClassVar[PreemptibleSpec.PreemptionPolicy]
        STOP: _ClassVar[PreemptibleSpec.PreemptionPolicy]
    UNSPECIFIED: PreemptibleSpec.PreemptionPolicy
    STOP: PreemptibleSpec.PreemptionPolicy
    ON_PREEMPTION_FIELD_NUMBER: _ClassVar[int]
    PRIORITY_FIELD_NUMBER: _ClassVar[int]
    on_preemption: PreemptibleSpec.PreemptionPolicy
    priority: int
    def __init__(self, on_preemption: _Optional[_Union[PreemptibleSpec.PreemptionPolicy, str]] = ..., priority: _Optional[int] = ...) -> None: ...

class ResourcesSpec(_message.Message):
    __slots__ = ["platform", "preset"]
    PLATFORM_FIELD_NUMBER: _ClassVar[int]
    PRESET_FIELD_NUMBER: _ClassVar[int]
    platform: str
    preset: str
    def __init__(self, platform: _Optional[str] = ..., preset: _Optional[str] = ...) -> None: ...

class InstanceGpuClusterSpec(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class AttachedDiskSpec(_message.Message):
    __slots__ = ["attach_mode", "existing_disk", "device_id"]
    class AttachMode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        UNSPECIFIED: _ClassVar[AttachedDiskSpec.AttachMode]
        READ_ONLY: _ClassVar[AttachedDiskSpec.AttachMode]
        READ_WRITE: _ClassVar[AttachedDiskSpec.AttachMode]
    UNSPECIFIED: AttachedDiskSpec.AttachMode
    READ_ONLY: AttachedDiskSpec.AttachMode
    READ_WRITE: AttachedDiskSpec.AttachMode
    ATTACH_MODE_FIELD_NUMBER: _ClassVar[int]
    EXISTING_DISK_FIELD_NUMBER: _ClassVar[int]
    DEVICE_ID_FIELD_NUMBER: _ClassVar[int]
    attach_mode: AttachedDiskSpec.AttachMode
    existing_disk: ExistingDisk
    device_id: str
    def __init__(self, attach_mode: _Optional[_Union[AttachedDiskSpec.AttachMode, str]] = ..., existing_disk: _Optional[_Union[ExistingDisk, _Mapping]] = ..., device_id: _Optional[str] = ...) -> None: ...

class ExistingDisk(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class ExistingFilesystem(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class AttachedFilesystemSpec(_message.Message):
    __slots__ = ["attach_mode", "mount_tag", "existing_filesystem"]
    class AttachMode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        UNSPECIFIED: _ClassVar[AttachedFilesystemSpec.AttachMode]
        READ_ONLY: _ClassVar[AttachedFilesystemSpec.AttachMode]
        READ_WRITE: _ClassVar[AttachedFilesystemSpec.AttachMode]
    UNSPECIFIED: AttachedFilesystemSpec.AttachMode
    READ_ONLY: AttachedFilesystemSpec.AttachMode
    READ_WRITE: AttachedFilesystemSpec.AttachMode
    ATTACH_MODE_FIELD_NUMBER: _ClassVar[int]
    MOUNT_TAG_FIELD_NUMBER: _ClassVar[int]
    EXISTING_FILESYSTEM_FIELD_NUMBER: _ClassVar[int]
    attach_mode: AttachedFilesystemSpec.AttachMode
    mount_tag: str
    existing_filesystem: ExistingFilesystem
    def __init__(self, attach_mode: _Optional[_Union[AttachedFilesystemSpec.AttachMode, str]] = ..., mount_tag: _Optional[str] = ..., existing_filesystem: _Optional[_Union[ExistingFilesystem, _Mapping]] = ...) -> None: ...

class InstanceStatus(_message.Message):
    __slots__ = ["state", "network_interfaces", "reconciling", "maintenance_event_id", "infiniband_topology_path"]
    class InstanceState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        UNSPECIFIED: _ClassVar[InstanceStatus.InstanceState]
        CREATING: _ClassVar[InstanceStatus.InstanceState]
        UPDATING: _ClassVar[InstanceStatus.InstanceState]
        STARTING: _ClassVar[InstanceStatus.InstanceState]
        RUNNING: _ClassVar[InstanceStatus.InstanceState]
        STOPPING: _ClassVar[InstanceStatus.InstanceState]
        STOPPED: _ClassVar[InstanceStatus.InstanceState]
        DELETING: _ClassVar[InstanceStatus.InstanceState]
        ERROR: _ClassVar[InstanceStatus.InstanceState]
    UNSPECIFIED: InstanceStatus.InstanceState
    CREATING: InstanceStatus.InstanceState
    UPDATING: InstanceStatus.InstanceState
    STARTING: InstanceStatus.InstanceState
    RUNNING: InstanceStatus.InstanceState
    STOPPING: InstanceStatus.InstanceState
    STOPPED: InstanceStatus.InstanceState
    DELETING: InstanceStatus.InstanceState
    ERROR: InstanceStatus.InstanceState
    STATE_FIELD_NUMBER: _ClassVar[int]
    NETWORK_INTERFACES_FIELD_NUMBER: _ClassVar[int]
    RECONCILING_FIELD_NUMBER: _ClassVar[int]
    MAINTENANCE_EVENT_ID_FIELD_NUMBER: _ClassVar[int]
    INFINIBAND_TOPOLOGY_PATH_FIELD_NUMBER: _ClassVar[int]
    state: InstanceStatus.InstanceState
    network_interfaces: _containers.RepeatedCompositeFieldContainer[_network_interface_pb2.NetworkInterfaceStatus]
    reconciling: bool
    maintenance_event_id: str
    infiniband_topology_path: InstanceStatusInfinibandTopologyPath
    def __init__(self, state: _Optional[_Union[InstanceStatus.InstanceState, str]] = ..., network_interfaces: _Optional[_Iterable[_Union[_network_interface_pb2.NetworkInterfaceStatus, _Mapping]]] = ..., reconciling: bool = ..., maintenance_event_id: _Optional[str] = ..., infiniband_topology_path: _Optional[_Union[InstanceStatusInfinibandTopologyPath, _Mapping]] = ...) -> None: ...

class InstanceStatusInfinibandTopologyPath(_message.Message):
    __slots__ = ["path"]
    PATH_FIELD_NUMBER: _ClassVar[int]
    path: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, path: _Optional[_Iterable[str]] = ...) -> None: ...



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class PostgresqlConfig16(_message.Message):
    __slots__ = ["autovacuum_work_mem", "statement_timeout", "idle_in_transaction_session_timeout", "autovacuum_vacuum_cost_delay", "autovacuum_vacuum_cost_limit", "autovacuum_naptime", "autovacuum_vacuum_scale_factor", "autovacuum_analyze_scale_factor", "default_transaction_read_only", "search_path", "max_connections", "shared_buffers"]
    AUTOVACUUM_WORK_MEM_FIELD_NUMBER: _ClassVar[int]
    STATEMENT_TIMEOUT_FIELD_NUMBER: _ClassVar[int]
    IDLE_IN_TRANSACTION_SESSION_TIMEOUT_FIELD_NUMBER: _ClassVar[int]
    AUTOVACUUM_VACUUM_COST_DELAY_FIELD_NUMBER: _ClassVar[int]
    AUTOVACUUM_VACUUM_COST_LIMIT_FIELD_NUMBER: _ClassVar[int]
    AUTOVACUUM_NAPTIME_FIELD_NUMBER: _ClassVar[int]
    AUTOVACUUM_VACUUM_SCALE_FACTOR_FIELD_NUMBER: _ClassVar[int]
    AUTOVACUUM_ANALYZE_SCALE_FACTOR_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_TRANSACTION_READ_ONLY_FIELD_NUMBER: _ClassVar[int]
    SEARCH_PATH_FIELD_NUMBER: _ClassVar[int]
    MAX_CONNECTIONS_FIELD_NUMBER: _ClassVar[int]
    SHARED_BUFFERS_FIELD_NUMBER: _ClassVar[int]
    autovacuum_work_mem: int
    statement_timeout: int
    idle_in_transaction_session_timeout: int
    autovacuum_vacuum_cost_delay: int
    autovacuum_vacuum_cost_limit: int
    autovacuum_naptime: int
    autovacuum_vacuum_scale_factor: float
    autovacuum_analyze_scale_factor: float
    default_transaction_read_only: bool
    search_path: str
    max_connections: int
    shared_buffers: int
    def __init__(self, autovacuum_work_mem: _Optional[int] = ..., statement_timeout: _Optional[int] = ..., idle_in_transaction_session_timeout: _Optional[int] = ..., autovacuum_vacuum_cost_delay: _Optional[int] = ..., autovacuum_vacuum_cost_limit: _Optional[int] = ..., autovacuum_naptime: _Optional[int] = ..., autovacuum_vacuum_scale_factor: _Optional[float] = ..., autovacuum_analyze_scale_factor: _Optional[float] = ..., default_transaction_read_only: bool = ..., search_path: _Optional[str] = ..., max_connections: _Optional[int] = ..., shared_buffers: _Optional[int] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v1/token_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!nebius/iam/v1/token_service.proto\x12\rnebius.iam.v1\x1a\x18nebius/annotations.proto\"\xdf\x02\n\x14\x45xchangeTokenRequest\x12\x1d\n\ngrant_type\x18\x01 \x01(\tR\tgrantType\x12\x30\n\x14requested_token_type\x18\x02 \x01(\tR\x12requestedTokenType\x12(\n\rsubject_token\x18\x03 \x01(\tB\x03\xc8J\x01R\x0csubjectToken\x12,\n\x12subject_token_type\x18\x04 \x01(\tR\x10subjectTokenType\x12\x16\n\x06scopes\x18\x05 \x03(\tR\x06scopes\x12\x1a\n\x08\x61udience\x18\x06 \x01(\tR\x08\x61udience\x12$\n\x0b\x61\x63tor_token\x18\x07 \x01(\tB\x03\xc8J\x01R\nactorToken\x12(\n\x10\x61\x63tor_token_type\x18\x08 \x01(\tR\x0e\x61\x63torTokenType\x12\x1a\n\x08resource\x18\t \x03(\tR\x08resource\"\xbf\x01\n\x13\x43reateTokenResponse\x12&\n\x0c\x61\x63\x63\x65ss_token\x18\x01 \x01(\tB\x03\xc8J\x01R\x0b\x61\x63\x63\x65ssToken\x12*\n\x11issued_token_type\x18\x02 \x01(\tR\x0fissuedTokenType\x12\x1d\n\ntoken_type\x18\x03 \x01(\tR\ttokenType\x12\x1d\n\nexpires_in\x18\x04 \x01(\x03R\texpiresIn\x12\x16\n\x06scopes\x18\x05 \x03(\tR\x06scopesBX\n\x14\x61i.nebius.pub.iam.v1B\x11TokenServiceProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v1.token_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v1B\021TokenServiceProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v1'
  _EXCHANGETOKENREQUEST.fields_by_name['subject_token']._options = None
  _EXCHANGETOKENREQUEST.fields_by_name['subject_token']._serialized_options = b'\310J\001'
  _EXCHANGETOKENREQUEST.fields_by_name['actor_token']._options = None
  _EXCHANGETOKENREQUEST.fields_by_name['actor_token']._serialized_options = b'\310J\001'
  _CREATETOKENRESPONSE.fields_by_name['access_token']._options = None
  _CREATETOKENRESPONSE.fields_by_name['access_token']._serialized_options = b'\310J\001'
  _globals['_EXCHANGETOKENREQUEST']._serialized_start=79
  _globals['_EXCHANGETOKENREQUEST']._serialized_end=430
  _globals['_CREATETOKENRESPONSE']._serialized_start=433
  _globals['_CREATETOKENRESPONSE']._serialized_end=624
# @@protoc_insertion_point(module_scope)



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class FederatedCredentials(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: FederatedCredentialsSpec
    status: FederatedCredentialsStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[FederatedCredentialsSpec, _Mapping]] = ..., status: _Optional[_Union[FederatedCredentialsStatus, _Mapping]] = ...) -> None: ...

class FederatedCredentialsSpec(_message.Message):
    __slots__ = ["oidc_provider", "federated_subject_id", "subject_id"]
    OIDC_PROVIDER_FIELD_NUMBER: _ClassVar[int]
    FEDERATED_SUBJECT_ID_FIELD_NUMBER: _ClassVar[int]
    SUBJECT_ID_FIELD_NUMBER: _ClassVar[int]
    oidc_provider: OidcCredentialsProvider
    federated_subject_id: str
    subject_id: str
    def __init__(self, oidc_provider: _Optional[_Union[OidcCredentialsProvider, _Mapping]] = ..., federated_subject_id: _Optional[str] = ..., subject_id: _Optional[str] = ...) -> None: ...

class OidcCredentialsProvider(_message.Message):
    __slots__ = ["issuer_url", "jwk_set_json"]
    ISSUER_URL_FIELD_NUMBER: _ClassVar[int]
    JWK_SET_JSON_FIELD_NUMBER: _ClassVar[int]
    issuer_url: str
    jwk_set_json: str
    def __init__(self, issuer_url: _Optional[str] = ..., jwk_set_json: _Optional[str] = ...) -> None: ...

class FederatedCredentialsStatus(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Invitation(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: InvitationSpec
    status: InvitationStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[InvitationSpec, _Mapping]] = ..., status: _Optional[_Union[InvitationStatus, _Mapping]] = ...) -> None: ...

class InvitationSpec(_message.Message):
    __slots__ = ["description", "email"]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    EMAIL_FIELD_NUMBER: _ClassVar[int]
    description: str
    email: str
    def __init__(self, description: _Optional[str] = ..., email: _Optional[str] = ...) -> None: ...

class InvitationStatus(_message.Message):
    __slots__ = ["tenant_user_account_id", "expires_at", "state"]
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        UNSPECIFIED: _ClassVar[InvitationStatus.State]
        CREATING: _ClassVar[InvitationStatus.State]
        CREATED: _ClassVar[InvitationStatus.State]
        PENDING: _ClassVar[InvitationStatus.State]
        EXPIRED: _ClassVar[InvitationStatus.State]
        ACCEPTED: _ClassVar[InvitationStatus.State]
    UNSPECIFIED: InvitationStatus.State
    CREATING: InvitationStatus.State
    CREATED: InvitationStatus.State
    PENDING: InvitationStatus.State
    EXPIRED: InvitationStatus.State
    ACCEPTED: InvitationStatus.State
    TENANT_USER_ACCOUNT_ID_FIELD_NUMBER: _ClassVar[int]
    EXPIRES_AT_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    tenant_user_account_id: str
    expires_at: _timestamp_pb2.Timestamp
    state: InvitationStatus.State
    def __init__(self, tenant_user_account_id: _Optional[str] = ..., expires_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., state: _Optional[_Union[InvitationStatus.State, str]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/storage/v1/bucket_counters.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius.storage.v1 import base_pb2 as nebius_dot_storage_dot_v1_dot_base__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'nebius/storage/v1/bucket_counters.proto\x12\x11nebius.storage.v1\x1a\x1cnebius/storage/v1/base.proto\"\x99\x03\n\x15\x43urrentBucketCounters\x12\x36\n\x17simple_objects_quantity\x18\x01 \x01(\x03R\x15simpleObjectsQuantity\x12.\n\x13simple_objects_size\x18\x02 \x01(\x03R\x11simpleObjectsSize\x12<\n\x1amultipart_objects_quantity\x18\x03 \x01(\x03R\x18multipartObjectsQuantity\x12\x34\n\x16multipart_objects_size\x18\x04 \x01(\x03R\x14multipartObjectsSize\x12<\n\x1amultipart_uploads_quantity\x18\x05 \x01(\x03R\x18multipartUploadsQuantity\x12\x36\n\x17inflight_parts_quantity\x18\x06 \x01(\x03R\x15inflightPartsQuantity\x12.\n\x13inflight_parts_size\x18\x07 \x01(\x03R\x11inflightPartsSize\"\xf6\x01\n\x18NonCurrentBucketCounters\x12\x36\n\x17simple_objects_quantity\x18\x01 \x01(\x03R\x15simpleObjectsQuantity\x12.\n\x13simple_objects_size\x18\x02 \x01(\x03R\x11simpleObjectsSize\x12<\n\x1amultipart_objects_quantity\x18\x03 \x01(\x03R\x18multipartObjectsQuantity\x12\x34\n\x16multipart_objects_size\x18\x04 \x01(\x03R\x14multipartObjectsSize\"\xfb\x01\n\x0e\x42ucketCounters\x12\x44\n\rstorage_class\x18\x01 \x01(\x0e\x32\x1f.nebius.storage.v1.StorageClassR\x0cstorageClass\x12\x44\n\x08\x63ounters\x18\x02 \x01(\x0b\x32(.nebius.storage.v1.CurrentBucketCountersR\x08\x63ounters\x12]\n\x14non_current_counters\x18\x03 \x01(\x0b\x32+.nebius.storage.v1.NonCurrentBucketCountersR\x12nonCurrentCountersBb\n\x18\x61i.nebius.pub.storage.v1B\x13\x42ucketCountersProtoP\x01Z/github.com/nebius/gosdk/proto/nebius/storage/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.storage.v1.bucket_counters_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030ai.nebius.pub.storage.v1B\023BucketCountersProtoP\001Z/github.com/nebius/gosdk/proto/nebius/storage/v1'
  _globals['_CURRENTBUCKETCOUNTERS']._serialized_start=93
  _globals['_CURRENTBUCKETCOUNTERS']._serialized_end=502
  _globals['_NONCURRENTBUCKETCOUNTERS']._serialized_start=505
  _globals['_NONCURRENTBUCKETCOUNTERS']._serialized_end=751
  _globals['_BUCKETCOUNTERS']._serialized_start=754
  _globals['_BUCKETCOUNTERS']._serialized_end=1005
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v1/tenant_user_account.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.iam.v1 import user_account_pb2 as nebius_dot_iam_dot_v1_dot_user__account__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'nebius/iam/v1/tenant_user_account.proto\x12\rnebius.iam.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a\x18nebius/annotations.proto\x1a nebius/iam/v1/user_account.proto\"\xe9\x01\n\x11TenantUserAccount\x12\x46\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\x06\xbaH\x03\xc8\x01\x01R\x08metadata\x12@\n\x04spec\x18\x02 \x01(\x0b\x32$.nebius.iam.v1.TenantUserAccountSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec\x12\x44\n\x06status\x18\x03 \x01(\x0b\x32&.nebius.iam.v1.TenantUserAccountStatusB\x04\xbaJ\x01\x05R\x06status:\x04\xbaJ\x01\x02\"\xf8\x01\n\x1fTenantUserAccountWithAttributes\x12P\n\x13tenant_user_account\x18\x01 \x01(\x0b\x32 .nebius.iam.v1.TenantUserAccountR\x11tenantUserAccount\x12?\n\nattributes\x18\x02 \x01(\x0b\x32\x1d.nebius.iam.v1.UserAttributesH\x00R\nattributes\x12,\n\x05\x65rror\x18\x03 \x01(\x0b\x32\x14.nebius.iam.v1.ErrorH\x00R\x05\x65rrorB\x14\n\x12\x61ttributesOptional\"\x89\x06\n\x0eUserAttributes\x12\x1a\n\x03sub\x18\x14 \x01(\tB\x03\xc0J\x01H\x00R\x03sub\x88\x01\x01\x12\x1c\n\x04name\x18\x01 \x01(\tB\x03\xc0J\x01H\x01R\x04name\x88\x01\x01\x12\'\n\ngiven_name\x18\x02 \x01(\tB\x03\xc0J\x01H\x02R\tgivenName\x88\x01\x01\x12)\n\x0b\x66\x61mily_name\x18\x03 \x01(\tB\x03\xc0J\x01H\x03R\nfamilyName\x88\x01\x01\x12\x37\n\x12preferred_username\x18\x04 \x01(\tB\x03\xc0J\x01H\x04R\x11preferredUsername\x88\x01\x01\x12\"\n\x07picture\x18\x05 \x01(\tB\x03\xc0J\x01H\x05R\x07picture\x88\x01\x01\x12u\n\x05\x65mail\x18\x06 \x01(\tBZ\xbaHT\xba\x01Q\n\x0cstring.email\x12#value must be a valid email address\x1a\x1cthis == \'\' || this.isEmail()\xc0J\x01H\x06R\x05\x65mail\x88\x01\x01\x12.\n\x0e\x65mail_verified\x18\x07 \x01(\x08\x42\x02\x18\x01H\x07R\remailVerified\x88\x01\x01\x12&\n\x08zoneinfo\x18\x08 \x01(\tB\x05\x18\x01\xc0J\x01H\x08R\x08zoneinfo\x88\x01\x01\x12 \n\x06locale\x18\t \x01(\tB\x03\xc0J\x01H\tR\x06locale\x88\x01\x01\x12+\n\x0cphone_number\x18\n \x01(\tB\x03\xc0J\x01H\nR\x0bphoneNumber\x88\x01\x01\x12;\n\x15phone_number_verified\x18\x0b \x01(\x08\x42\x02\x18\x01H\x0bR\x13phoneNumberVerified\x88\x01\x01\x42\x06\n\x04_subB\x07\n\x05_nameB\r\n\x0b_given_nameB\x0e\n\x0c_family_nameB\x15\n\x13_preferred_usernameB\n\n\x08_pictureB\x08\n\x06_emailB\x11\n\x0f_email_verifiedB\x0b\n\t_zoneinfoB\t\n\x07_localeB\x0f\n\r_phone_numberB\x18\n\x16_phone_number_verified\"!\n\x05\x45rror\x12\x18\n\x07message\x18\x01 \x01(\tR\x07message\"\xb6\x01\n\x15TenantUserAccountSpec\x12\x65\n\x12visible_attributes\x18\x01 \x01(\x0b\x32\x36.nebius.iam.v1.TenantUserAccountSpec.VisibleAttributesR\x11visibleAttributes\x1a\x36\n\x11VisibleAttributes\x12!\n\tattribute\x18\x01 \x03(\tB\x03\xc0J\x01R\tattribute\"\xc4\x02\n\x17TenantUserAccountStatus\x12\x42\n\x05state\x18\x01 \x01(\x0e\x32,.nebius.iam.v1.TenantUserAccountStatus.StateR\x05state\x12#\n\rinvitation_id\x18\x02 \x01(\tR\x0cinvitationId\x12#\n\rfederation_id\x18\x03 \x01(\tR\x0c\x66\x65\x64\x65rationId\x12T\n\x12user_account_state\x18\x04 \x01(\x0e\x32&.nebius.iam.v1.UserAccountStatus.StateR\x10userAccountState\"E\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\n\n\x06\x41\x43TIVE\x10\x01\x12\x0c\n\x08INACTIVE\x10\x02\x12\x0b\n\x07\x42LOCKED\x10\x03\x42]\n\x14\x61i.nebius.pub.iam.v1B\x16TenantUserAccountProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v1.tenant_user_account_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v1B\026TenantUserAccountProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v1'
  _TENANTUSERACCOUNT.fields_by_name['metadata']._options = None
  _TENANTUSERACCOUNT.fields_by_name['metadata']._serialized_options = b'\272H\003\310\001\001'
  _TENANTUSERACCOUNT.fields_by_name['spec']._options = None
  _TENANTUSERACCOUNT.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _TENANTUSERACCOUNT.fields_by_name['status']._options = None
  _TENANTUSERACCOUNT.fields_by_name['status']._serialized_options = b'\272J\001\005'
  _TENANTUSERACCOUNT._options = None
  _TENANTUSERACCOUNT._serialized_options = b'\272J\001\002'
  _USERATTRIBUTES.fields_by_name['sub']._options = None
  _USERATTRIBUTES.fields_by_name['sub']._serialized_options = b'\300J\001'
  _USERATTRIBUTES.fields_by_name['name']._options = None
  _USERATTRIBUTES.fields_by_name['name']._serialized_options = b'\300J\001'
  _USERATTRIBUTES.fields_by_name['given_name']._options = None
  _USERATTRIBUTES.fields_by_name['given_name']._serialized_options = b'\300J\001'
  _USERATTRIBUTES.fields_by_name['family_name']._options = None
  _USERATTRIBUTES.fields_by_name['family_name']._serialized_options = b'\300J\001'
  _USERATTRIBUTES.fields_by_name['preferred_username']._options = None
  _USERATTRIBUTES.fields_by_name['preferred_username']._serialized_options = b'\300J\001'
  _USERATTRIBUTES.fields_by_name['picture']._options = None
  _USERATTRIBUTES.fields_by_name['picture']._serialized_options = b'\300J\001'
  _USERATTRIBUTES.fields_by_name['email']._options = None
  _USERATTRIBUTES.fields_by_name['email']._serialized_options = b'\272HT\272\001Q\n\014string.email\022#value must be a valid email address\032\034this == \'\' || this.isEmail()\300J\001'
  _USERATTRIBUTES.fields_by_name['email_verified']._options = None
  _USERATTRIBUTES.fields_by_name['email_verified']._serialized_options = b'\030\001'
  _USERATTRIBUTES.fields_by_name['zoneinfo']._options = None
  _USERATTRIBUTES.fields_by_name['zoneinfo']._serialized_options = b'\030\001\300J\001'
  _USERATTRIBUTES.fields_by_name['locale']._options = None
  _USERATTRIBUTES.fields_by_name['locale']._serialized_options = b'\300J\001'
  _USERATTRIBUTES.fields_by_name['phone_number']._options = None
  _USERATTRIBUTES.fields_by_name['phone_number']._serialized_options = b'\300J\001'
  _USERATTRIBUTES.fields_by_name['phone_number_verified']._options = None
  _USERATTRIBUTES.fields_by_name['phone_number_verified']._serialized_options = b'\030\001'
  _TENANTUSERACCOUNTSPEC_VISIBLEATTRIBUTES.fields_by_name['attribute']._options = None
  _TENANTUSERACCOUNTSPEC_VISIBLEATTRIBUTES.fields_by_name['attribute']._serialized_options = b'\300J\001'
  _globals['_TENANTUSERACCOUNT']._serialized_start=181
  _globals['_TENANTUSERACCOUNT']._serialized_end=414
  _globals['_TENANTUSERACCOUNTWITHATTRIBUTES']._serialized_start=417
  _globals['_TENANTUSERACCOUNTWITHATTRIBUTES']._serialized_end=665
  _globals['_USERATTRIBUTES']._serialized_start=668
  _globals['_USERATTRIBUTES']._serialized_end=1445
  _globals['_ERROR']._serialized_start=1447
  _globals['_ERROR']._serialized_end=1480
  _globals['_TENANTUSERACCOUNTSPEC']._serialized_start=1483
  _globals['_TENANTUSERACCOUNTSPEC']._serialized_end=1665
  _globals['_TENANTUSERACCOUNTSPEC_VISIBLEATTRIBUTES']._serialized_start=1611
  _globals['_TENANTUSERACCOUNTSPEC_VISIBLEATTRIBUTES']._serialized_end=1665
  _globals['_TENANTUSERACCOUNTSTATUS']._serialized_start=1668
  _globals['_TENANTUSERACCOUNTSTATUS']._serialized_end=1992
  _globals['_TENANTUSERACCOUNTSTATUS_STATE']._serialized_start=1923
  _globals['_TENANTUSERACCOUNTSTATUS_STATE']._serialized_end=1992
# @@protoc_insertion_point(module_scope)
